import os
import sys
import subprocess
import csv

def check_command(cmdname):
    '''
    Checks if command is present and exits if no
    '''
    if not os.path.exists(cmdname):
        print 'No such command (%s)' % (cmdname,)
        sys.exit(1)

def run_command(cmd, std_in=None):
    '''
    Runs an external command and returns result
    '''
    if not type(cmd) is list:
        return None
    if not std_in:
        stdin_arg = open('/dev/null')
    else:
        stdin_arg = subprocess.PIPE
    output = subprocess.Popen(
        cmd,
        stdin=stdin_arg,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        close_fds=True)
    if not std_in:
        std_out, std_err = output.communicate()
    else:
        std_out, std_err = output.communicate(std_in)
    if output.returncode != 0:
        print 'ERROR: command failed:\n%s' % (
            ((std_err and std_err) or std_out),)
        sys.exit(output.returncode)
    return std_out


def print_text(data, title='id', sort_key=str):
    '''
    Prints data as pure text
    '''
    data_keys = data.keys()
    if len(data_keys) == 0:
        return
    fields = map((lambda x: x[0]), data[data_keys[0]])
    fmt_list = ['%32s']
    fmt_list.extend(['%16s'] * (len(fields)))
    fmt = ''.join(fmt_list)
    print fmt % tuple([title] + fields)
    print '-' * 80
    for item in sorted(data_keys, key=sort_key):
        row = [item]
        row.extend(map((lambda x:x[1]), data[item]))
        print fmt % tuple(row)

def print_csv(data, title='id', sort_key=str):
    '''
    Prints data as comma separated values
    '''
    data_keys = data.keys()
    if len(data_keys) == 0:
        return
    fields = map((lambda x: x[0]), data[data_keys[0]])
    fields = [title] + fields
    csv_out = csv.writer(sys.stdout, quoting=csv.QUOTE_MINIMAL)
    fmt = ','.join(['%s'] * len(fields))
    print fmt % tuple(fields)
    for item in sorted(data_keys, key=sort_key):
        row = [item]
        row.extend(map((lambda x:x[1]), data[item]))
        csv_out.writerow(row)
        
def print_text_error(message):
    print "ERROR:%s" % (message)
    
def print_csv_error(message):
        csv_out = csv.writer(sys.stdout, quoting=csv.QUOTE_ALL)
        csv_out.writerow(['ERROR', message])
