#!/bin/sh
eval 'if [ -x /usr/local/cpanel/3rdparty/bin/perl ]; then exec /usr/local/cpanel/3rdparty/bin/perl -x -- $0 ${1+"$@"}; else exec /usr/bin/perl -x -- $0 ${1+"$@"};fi'
if 0;
#!/usr/bin/perl
# CloudLinux LVE Manager VERSION:0.83

BEGIN { unshift @INC, "/usr/local/cpanel"; }

use strict;
use Cpanel::Form    ();
use Cpanel::SafeRun ();
use Data::Dumper    ();

my %FORM    = Cpanel::Form::parseform();
my $user    = $ENV{'REMOTE_USER'}||$ARGV[0];

my $action = exists $FORM{action} ? $FORM{action} : 'extlist';

my %dispatchTable = (
    extlist     => \&getExtensionList,
    optlist     => \&getOptionList,
    extsave     => \&saveExtensionSet,
    optsave     => \&saveOptionSet,
    extdefaults => \&extensionsToDefaults,
);

$dispatchTable{$action}->() || $dispatchTable{extlist}->();

sub getExtensionList {
    my %data;
    my ( $version, $status, $message ) = ( $FORM{version}, 'OK', [] );
    my @params = (
        '/usr/bin/cl-selector',
        '--list-extensions=php',
        "--user=$user",
        "--version=$version",
        '--all',
    );
    my $result = Cpanel::SafeRun::Errors::saferunallerrors(@params);
    for my $line ( split( /\n/, $result ) ) {
        if ( index( $line, 'ERROR:' ) != -1) {
            $status = 'ERROR';
            push @{ $message }, substr( $line, 6 );
            last;
        }
        if ( index( $line, 'WARN:' ) != -1) {
            $status = 'WARN';
            push @{ $message }, substr( $line, 5 );
            next;
        }
        my ( $status, $extension ) = split /\s+/, $line;
        $status = $status eq '+' ? 1 : 0;
        $data{$extension} = $status;
    }
    my $output = {
        status => $status,
        data   => [ map +{ title=>$_, status=>$data{$_} }, sort keys %data ],
    };
    $output->{message} = $message if @{ $message };
    my $d = Data::Dumper->new([ $output ]);
    print "Content-Type: application/json\n\n";
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
}

sub getOptionList {
    my ( $version, $status, $message ) = ( $FORM{version}, 'OK', [] );
    my @params = (
        '/usr/bin/piniset',
        "--user=$user",
        "--version=$version",
        '--json',
    );
    print "Content-Type: application/json\n\n";
    print Cpanel::SafeRun::Errors::saferunallerrors(@params);
}

sub saveExtensionSet {
    my ( $version, $extset, $status, $message ) = ( $FORM{version}, $FORM{extset}, 'OK', [] );
    my %exthash = map { $_ => 1 } split( /,/, $extset );
    if ( exists $FORM{version} ) {
        $version = $FORM{version};
    }
    my @params = (
        '/usr/bin/cl-selector',
        '--interpreter=php',
        "--update=$extset",
        "--user=$user",
        "--version=$version"
    );
    
    my $result = Cpanel::SafeRun::Errors::saferunallerrors(@params);
    
    if ( $result ) {
        for my $line ( split( /\n/, $result ) ) {
            if ( index( $line, 'ERROR:' ) != -1) {
                $status = 'ERROR';
                push @{ $message }, substr( $line, 6 );
                last;
            }
            if ( index( $line, 'WARN:' ) != -1) {
                $status = 'WARN';
                push @{ $message }, substr( $line, 5 );
                if ( $line =~ /WARN:(\S+)\s(\S+).*?(?:\((.*)\))?\.?$/ ) {
                    my ( $ext, $action, $arg ) = ( $1, $2, $3 );
                    if ( $action eq 'skipped' ) {
                        $exthash{$ext} = 0;
                    }
                    elsif ( $action eq 'enabled' ) {
                        $exthash{$ext} = 1
                    }
                    elsif ( $action eq 'left' ) {
                        $exthash{$ext} = 1
                    }
                }
            }
        }
    }
    my $output = {
        status => $status,
        data   => [ map +{ title=>$_, status=>$exthash{$_} }, sort keys %exthash ],
    };
    $output->{message} = $message if @{ $message };
    my $d = Data::Dumper->new([ $output ]);
    print "Content-Type: application/json\n\n";
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
}

sub saveOptionSet {
    my ( $version, $optset, $status, $message ) = ( $FORM{version}, $FORM{optset}, 'OK', [] );
    open my $t, '>', '/tmp/phpinfo';
    print $t $optset;
    close $t;
    my @params = (
        '/usr/bin/piniset',
        "--user=$user",
        "--version=$version",
        "--replace=$optset",
        '--base64',
        '--json'
    );
    print "Content-Type: application/json\n\n";
    print Cpanel::SafeRun::Errors::saferunallerrors(@params);
}

sub extensionsToDefaults {
    my %exthash = ();
    my %buffer = ();
    my ( $version, $status, $message ) = ( $FORM{version}, 'OK', [] );
    my @params = (
        '/usr/bin/cl-selector',
        '--set-default-extensions=php',
        "--user=$user",
        "--version=$version",
    );
    my $result = Cpanel::SafeRun::Errors::saferunallerrors(@params);

    if ( $result ) {
        for my $line ( split( /\n/, $result ) ) {
            if ( index( $line, 'ERROR:' ) != -1) {
                $status = 'ERROR';
                push @{ $message }, substr( $line, 6 );
                last;
            }
            if ( index( $line, 'WARN:' ) != -1) {
                $status = 'WARN';
                push @{ $message }, substr( $line, 5 );
                if ( $line =~ /WARN:(\S+)\s(\S+).*?(?:\((.*)\))?\.?$/ ) {
                    my ( $ext, $action, $arg ) = ( $1, $2, $3 );
                    if ( $action eq 'skipped' ) {
                        $buffer{$ext} = 0;
                    }
                    elsif ( $action eq 'enabled' ) {
                        $buffer{$ext} = 1
                    }
                    elsif ( $action eq 'left' ) {
                        $buffer{$ext} = 1
                    }
                }
                next;
            }
            else {
                %exthash = map { $_ => 1 } split( /,/, $line );
            }
        }
    }
    
    for my $ext ( keys %buffer ) {
        if ( $buffer{$ext} == 1 ) {
            $exthash{$ext} = 1;
        }
        elsif ( $buffer{$ext} == 0 ) {
            delete $exthash{$ext} if exists $exthash{$ext};
        }
    }
    my $output = {
        status => $status,
        data   => [ map +{ title=>$_, status=>$exthash{$_} }, sort keys %exthash ],
    };
    $output->{message} = $message if @{ $message };
    my $d = Data::Dumper->new([ $output ]);
    print "Content-Type: application/json\n\n";
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
}
