package Cpanel::LVEInfo;

# cpanel - Cpanel/LVEInfo.pm                  Copyright(c) 2011 CloudLinux, Inc.
#                                                           All Rights Reserved.
# info@cloudlinux.com                                  http://www.cloudlinux.com
# This module is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself. See http://dev.perl.org/licenses/artistic.html
# CloudLinux LVE Manager VERSION:0.83

use Cpanel::Math     ();
use Cpanel::StatsBar ();

my @lve;
my @def;
my @quota;

sub get_lve_info {
    my ($id) = @_;
    @quota = @{ get_quota_info($id) };
    if ( open my $fh, '<', '/var/lve/info' ) {
        while ( my $line = readline $fh ) {
            chomp $line;
            my @vals = split( /,/, $line );
            if ( $vals[0] == $id ) {
                close $fh;
                return @vals;
            }
            elsif ( $vals[0] eq '0' ) {
                @def = @vals;
            }
        }
        close $fh;
    }
    return;
}

sub get_quota_info {
    my ($id) = @_;
    my @params = ('/usr/bin/cl-quota', "--user-id=$id", '--csv');
    my $rv = Cpanel::SafeRun::Errors::saferunallerrors(@params);
    my ($header, $data) = split(/\n/, $rv, 2);
    chomp $data;
    return [split(/,/, $data)];
}

sub api2_getSummary {
    my %OPTS = @_;
    my $inodes = $OPTS{'lveinodes'};
    my @fields = qw/id mep lep cpu_usage lcpu mem_usage lmem mem_fault
                    mep_fault lmemphy memphy memphy_fault lnproc nproc
                    nproc_fault lcpuw iousage iolimit/;
 
    my %data = (
       cpu   => { t => 'CPU Usage', u => 'cpu_usage', l => 'lcpu' },
       mep   => { t => 'Entry Processes', u => 'mep', l => 'lep', f => 'mep_fault' },
       vmem  => { t => 'Virtual Memory Usage', u => 'mem_usage', l => 'lmem', f => 'mem_fault' },
       pmem  => { t => 'Physical Memory Usage', u => 'memphy', l => 'lmemphy', f => 'memphy_fault' },
       nproc => { t => 'Number of processes', u => 'nproc', l => 'lnproc', f => 'nproc_fault' },
       io    => { t => 'I/O usage', u => 'iousage', l => 'iolimit' },
    );
    
    my $doRoundK = sub {
        my $v = shift;
        if ( $v < 1024 ) {
            return sprintf "%0.2fk", $v;
        }
        elsif ( $v >= 1024 and $v < 1048576 ) {
            return sprintf "%0.2fM", $v / 1024;
        }
        else {
            return sprintf "%0.2fG", $v / 1048576;
        }
    };
    if ( open my $fh, '<', '/var/lve/info' ) {
        my %vals = ();
        while ( my $line = readline $fh ) {
            chomp $line;
            my @v = split( /,/, $line );
            my $key = $v[0] == 0 ? 'default' : $v[0];
            for my $i ( 1 .. $#v ) {
                $vals{$key}->{ $fields[$i] } = $v[$i];
            }
        }
        close $fh;
        my $user = getuid();
        if ($inodes) {
            $data{inodes} = { t => 'inodes usage', u => 'inodesusage', l => 'inodeslimit' };
            my @quota = @{ get_quota_info($user) };
            for my $key ( keys %vals ) {
                $vals{$key}->{inodesusage} = $quota[1];
                $vals{$key}->{inodeslimit} = $quota[2];
            }
        }
        my $arref = [];
        my $keyname = ( defined $user and exists $vals{$user} ) ? $user : 'default';

        for my $key ( sort keys %data ) {
            #next if $vals{$keyname}->{ $data{$key}->{l} } == 0;
            push @{ $arref },
                {
                    title=>$data{$key}->{t},
                    usage=>(($key eq 'pmem' or $key eq 'vmem')
							? $doRoundK->( $vals{$keyname}->{ $data{$key}->{u} } * 4 )
							: ($key eq 'cpu') ? sprintf( "%.1f", ( $vals{$keyname}->{ $data{$key}->{u} } * 100 / ( $vals{$keyname}->{ $data{$key}->{l} } || 1 ) ) )
								: $vals{$keyname}->{ $data{$key}->{u} } )
                        			.($key eq 'cpu' ? '%' : ''),
                    limit=>(($key eq 'pmem' or $key eq 'vmem') ? $doRoundK->( $vals{$keyname}->{ $data{$key}->{l} } * 4 ) : ($key eq 'cpu') ? 100 : $vals{$keyname}->{ $data{$key}->{l} })
                        .($key eq 'cpu' ? '%' : ''),
                    faults=>(exists $data{$key}->{f} ? $vals{$keyname}->{ $data{$key}->{f} } : '-'),
                } unless $vals{$keyname}->{ $data{$key}->{l} } == 0;
        }
        return $arref;
    }
    return;
}

sub LVEInfo_init {

    my $uid = getuid();
    @lve = get_lve_info($uid);
    
    Cpanel::StatsBar::_load_stats_ref();
    $Cpanel::StatsBar::rSTATS->{'lvecpu' }{'module'}  = 'LVEInfo';
    $Cpanel::StatsBar::rSTATS->{'lvecpu' }{'_count'}  = \&Cpanel::LVEInfo::_cpu;
    $Cpanel::StatsBar::rSTATS->{'lvecpu' }{'_max'}    = '100 %';
    $Cpanel::StatsBar::rSTATS->{'lvecpu' }{'langkey'} = 'CPU Usage';
    #$Cpanel::StatsBar::rSTATS->{'lvecpu' }{'units'}   = '%';
    
	unless ( _is_mem_unlimited() ) {
        $Cpanel::StatsBar::rSTATS->{'lvemem' }{'_count'}  = \&Cpanel::LVEInfo::_mem;
        $Cpanel::StatsBar::rSTATS->{'lvemem' }{'_max'}    = \&Cpanel::LVEInfo::_mem_limit;
        $Cpanel::StatsBar::rSTATS->{'lvemem' }{'langkey'} = 'Virtual Memory Usage';
        #$Cpanel::StatsBar::rSTATS->{'lvemem' }{'units'}   = 'KB';
        $Cpanel::StatsBar::rSTATS->{'lvemem' }{'module'}  = 'LVEInfo';
	}

    unless ( Cpanel::LVEInfo::_is_pmem_unlimited() ) {
        $Cpanel::StatsBar::rSTATS->{'lvepmem' }{'module'} = 'LVEInfo';
        $Cpanel::StatsBar::rSTATS->{'lvepmem'}{'_count'}  = \&Cpanel::LVEInfo::_pmem;
        $Cpanel::StatsBar::rSTATS->{'lvepmem'}{'_max'}    = \&Cpanel::LVEInfo::_pmem_limit;
        $Cpanel::StatsBar::rSTATS->{'lvepmem'}{'langkey'} = 'Physical Memory Usage';
        #$Cpanel::StatsBar::rSTATS->{'lvepmem'}{'units'}   = 'KB';
	}

    unless ( Cpanel::LVEInfo::_is_mep_unlimited() ) {
        $Cpanel::StatsBar::rSTATS->{'lvemep' }{'module'}  = 'LVEInfo';
        $Cpanel::StatsBar::rSTATS->{'lvemep' }{'_count'}  = \&Cpanel::LVEInfo::_mep;
        $Cpanel::StatsBar::rSTATS->{'lvemep' }{'_max'}    = \&Cpanel::LVEInfo::_mep_limit;
        $Cpanel::StatsBar::rSTATS->{'lvemep' }{'langkey'} = 'Entry Processes';
    }

	unless ( Cpanel::LVEInfo::_is_iop_unlimited() ) {
        $Cpanel::StatsBar::rSTATS->{'lveiop' }{'module'}  = 'LVEInfo';
        $Cpanel::StatsBar::rSTATS->{'lveiop' }{'_count'}  = \&Cpanel::LVEInfo::_iop;
        $Cpanel::StatsBar::rSTATS->{'lveiop' }{'_max'}    = \&Cpanel::LVEInfo::_iop_limit;
        $Cpanel::StatsBar::rSTATS->{'lveiop' }{'langkey'} = 'I/O Usage';
        #$Cpanel::StatsBar::rSTATS->{'lveiop' }{'units'}   = 'KB/s';
	}

	unless ( Cpanel::LVEInfo::_is_proc_unlimited() ) {
        $Cpanel::StatsBar::rSTATS->{'lveproc'}{'module'}  = 'LVEInfo';
        $Cpanel::StatsBar::rSTATS->{'lveproc'}{'_count'}  = \&Cpanel::LVEInfo::_proc;
        $Cpanel::StatsBar::rSTATS->{'lveproc'}{'_max'}    = \&Cpanel::LVEInfo::_proc_limit;
        $Cpanel::StatsBar::rSTATS->{'lveproc'}{'langkey'} = 'Number of Processes';
	}
    
	unless ( is_file_usage_enabled() or Cpanel::LVEInfo::_is_quota_unlimited() ) {
        $Cpanel::StatsBar::rSTATS->{'lvequota' }{'module'}  = 'LVEInfo';
        $Cpanel::StatsBar::rSTATS->{'lvequota' }{'_count'}  = \&Cpanel::LVEInfo::_quota;
        $Cpanel::StatsBar::rSTATS->{'lvequota' }{'_max'}    = \&Cpanel::LVEInfo::_quota_limit;
        $Cpanel::StatsBar::rSTATS->{'lvequota' }{'langkey'} = 'inodes';
	}
}

# Format of line in /var/lve/info (formed by lvestats-server):
# (id, mep, lep, cpu_usage, lcpu, mem_usage, lmem,
# mem_fault, mep_fault, lmemphy, memphy, memphy_fault,
# lnproc, nproc, nproc_fault, lcpuw, ioread, iowrite)
use constant ID_INDEX     => 0;
use constant MEP_INDEX    => 1;
use constant LEP_INDEX    => 2;
use constant CPU_INDEX    => 3;
use constant LCPU_INDEX   => 4;
use constant MEM_INDEX    => 5;
use constant LMEM_INDEX   => 6;
use constant MEMF_INDEX   => 7;
use constant MEPF_INDEX   => 8;
use constant LPMEM_INDEX  => 9;
use constant PMEM_INDEX   => 10;
use constant PMEMF_INDEX  => 11;
use constant LNPROC_INDEX => 12;
use constant NPROC_INDEX  => 13;
use constant NPROCF_INDEX => 14;
use constant LCPUW_INDEX  => 15;
use constant IOUSAGE_INDEX => 16;
use constant IOLIMIT_INDEX => 17;

sub _mem {
    return k_to_m($lve[MEM_INDEX] << 2) if (@lve);
    return 0;
}

sub round_mem {
    return RoundMemK( $lve[MEM_INDEX] << 2 ) if (@lve);
    return 0;
}

sub LVEInfo_mem {
    print round_mem();
}

sub _mem_limit {
    return k_to_m($lve[LMEM_INDEX] << 2).' MB' if (@lve);
    return k_to_m($def[LMEM_INDEX] << 2).' MB';
}

sub _is_mem_unlimited {
    if (@lve) {
        return ($lve[LMEM_INDEX] == 0);
    }
    if (@def) {
        return ($def[LMEM_INDEX] == 0);
    }
    return 1;
}

sub round_mem_limit {
    return RoundMemK( $lve[LMEM_INDEX] << 2 ) if (@lve);
    return RoundMemK( $def[LMEM_INDEX] << 2 );
}

sub LVEInfo_mem_limit {
    print round_mem_limit();
}

sub _cpu {
    if   ( @lve && $lve[LCPU_INDEX] != 0 ) { return int( $lve[CPU_INDEX] / $lve[LCPU_INDEX] * 100 ); }
    else                                   { return 0; }
}

sub LVEInfo_cpu {
    print _cpu();
}

sub getuid() {
    my $user = $Cpanel::user;
    return ( Cpanel::PwCache::getpwnam($user) )[2];
}

sub LVEInfo_start {
}

sub _mep {
    if   (@lve) { return $lve[MEP_INDEX]; }
    else        { return 0; }
}

sub LVEInfo_mep {
    print _mep();
}

sub _mep_limit {
    if   (@lve) { return $lve[LEP_INDEX]; }
    else        { return $def[LEP_INDEX]; }
}

sub _is_mep_unlimited {
    if (@lve) {
        return ($lve[LEP_INDEX] == 0);
    }
    if (@def) {
        return ($def[LEP_INDEX] == 0);
    }
    return 1;
}

sub LVEInfo_mep_limit {
    print _mep_limit();
}

sub _pmem {
    return k_to_m($lve[PMEM_INDEX] << 2) if (@lve);
    return 0;
}

sub LVEInfo_pmem {
    print _pmem();
}

sub _pmem_limit {
    return k_to_m($lve[LPMEM_INDEX] << 2).' MB' if (@lve);
    return k_to_m($def[LPMEM_INDEX] << 2).' MB';
}

sub LVEInfo_pmem_limit {
    print _pmem_limit();
}

sub _is_pmem_unlimited {
    if (@lve) {
        return ($lve[LPMEM_INDEX] == 0);
    }
    if (@def) {
        return ($def[LPMEM_INDEX] == 0);
    }
    return 1;
}

sub _proc {
    if   (@lve) { return $lve[NPROC_INDEX]; }
    else        { return 0; }
}

sub LVEInfo_proc {
    print _proc();
}

sub _proc_limit {
    if   (@lve) { return $lve[LNPROC_INDEX]; }
    else        { return $def[LNPROC_INDEX]; }
}

sub LVEInfo_proc_limit {
    print _proc_limit();
}

sub _is_proc_unlimited {
    if (@lve) {
        return ($lve[LNPROC_INDEX] == 0);
    }
    if (@def) {
        return ($def[LNPROC_INDEX] == 0);
    }
    return 1;
}

sub _iop {
    if   (@lve) { return k_to_m($lve[IOUSAGE_INDEX]); }
    else        { return 0; }
}

sub LVEInfo_iop {
    print _iop();
}

sub _iop_limit {
    if   (@lve) { return k_to_m($lve[IOLIMIT_INDEX]).' MB/s'; }
    else        { return k_to_m($def[IOLIMIT_INDEX]).' MB/s'; }
}

sub _is_iop_unlimited {
    if (@lve) {
        return ($lve[IOLIMIT_INDEX] == 0);
    }
    if (@def) {
        return ($def[IOLIMIT_INDEX] == 0);
    }
    return 1;
}

sub LVEInfo_iop_limit {
    print _iop_limit();
}

sub _quota {
    if   (@quota) { return $quota[1]; }
    else        { return 0; }
}

sub LVEInfo_quota {
    print _quota();
}

sub _quota_limit {
    if   (@quota) { return $quota[2]; }
    else        { return 0; }
}

sub _is_quota_unlimited {
    return 1 unless @quota;
    return 1 if $quota[2] == 0;
    return 0;
}

sub LVEInfo_quota_limit {
    print _quota_limit();
}

sub isError {
    my $x = shift;
    return $x ? 'errors' : 'lveok';
}

sub api2_getUsage2 {
    my %CFG = @_;
    my @RSD;
    my %attrs = (
        lEP     => [ qw/aEP mEP lEP/ ],
        lVMem    => [ qw/aVMem mVMem lVMem/ ],
        lPMem => [ qw/aPMem mPMem lPMem/ ],
        lNproc  => [ qw/aNproc mNproc lNproc/ ],
        lIO  => [ qw/aIO mIO lIO/ ],
    );
    my $lveid   = getuid();
    my $period  = !exists $CFG{'period'} || $CFG{'period'} eq '' ? '1d' : $CFG{'period'};
    my $INPUT   = Cpanel::SafeRun::Errors::saferunallerrors( '/usr/sbin/lveinfo', '--csv', "--period=$period", "--id=$lveid" );
    my @results = split( /\n+/, $INPUT );
    my @header = split( /,/, shift @results);
    push @RSD, { map { $_ => ($titles{$_} || '') } @header };
    my %limits = ( lEP => 0, lVMem => 0, lPMem => 0, lNproc => 0, lIO => 0 );
    for my $i (@results) {
        my @data = split( /,/, $i );
        $data[0] =~ s/\s/&nbsp;/;
        $data[1] =~ s/\s/&nbsp;/;
        my %struct = map { $header[$_] => (
                               $header[$_] =~ /(?:a|m)(?:v|p)mem/i
                                   ? sprintf("%0.2f",$data[$_]/1048576)
                                   : $header[$_] =~ /l(?:v|p)mem/i
                                       ? sprintf("%d",$data[$_]/1048576)
                                       : $data[$_]
                                          ) } 0 .. $#header;
        for my $k ( keys %limits ) {
            $limits{$k} += $struct{$k};
        }
        push @RSD, \%struct;
    }
    my @limits_keys = keys %limits;
    for my $i ( @RSD ) {
        for my $k ( @limits_keys ) {
            if ( $limits{$k} == 0 ) {
                for my $j ( @{ $attrs{$k} } ) {
                    delete $i->{$j};
                }
            }
        }
    }
    return \@RSD;
}

sub api2_getUsage {
    my %CFG = @_;
    my @RSD;

    my $lveid   = getuid();
    my $period  = !exists $CFG{'period'} || $CFG{'period'} eq '' ? '1d' : $CFG{'period'};
    my $INPUT   = Cpanel::SafeRun::Errors::saferunallerrors( '/usr/sbin/lveinfo', '--csv', "--period=$period", "--id=$lveid" );
    my @results = split( /\n+/, $INPUT );
    shift @results;

    foreach my $i (@results) {

        # From,To,aCPU,mCPU,lCPU,aEP,mEP,lEP,aMem,mMem,lMem,MemF,MepF
        # From,To,aCPU,mCPU,lCPU,aEP,mEP,lEP,aMem,mMem,lMem,MemF,MepF,lMemPhy,aMemPhy,mMemPhy,fMemPhy,lNproc,aNproc,mNproc,fNproc,lCPUw
        # From,To,aCPU,mCPU,lCPU,aEP,mEP,lEP,aVMem,mVMem,lVMem,aPMem,mPMem,lPMem,aNproc,mNproc,lNproc,EPf,VMemF,PMemF,NprocF
        my ( $from, $to, $acpu, $mcpu, $lcpu, $aep, $mep, $lep, $aMem, $mMem, $lMem, $memf, $mepf,
             $aPmem, $mPmem, $lPmem, $aNproc, $mNproc, $lNproc, $pmemf, $nprocf, $aIO, $mIO, $lIO ) = split( /,/, $i );
        my ( $apcpu, $mpcpu ) = ( 0, 0 );
        $lcpu  = 100 if ( $lcpu == 0 );
        $apcpu = Cpanel::Math::ceil( $acpu * 100 / $lcpu );
        $mpcpu = Cpanel::Math::ceil( $mcpu * 100 / $lcpu );

        unshift(
            @RSD,
            {
                'from'    => $from,
                'to'      => $to,
                'acpu'    => $acpu,
                'mcpu'    => $mcpu,
                'lcpu'    => $lcpu,
                'aep'     => $aep,
                'mep'     => $mep,
                'lep'     => $lep,
                'aMem'    => $aMem,
                'mMem'    => $mMem,
                'lMem'    => $lMem,
                'aPMem'   => $aPmem,
                'mPMem'   => $mPmem,
                'lPMem'   => $lPmem,
                'aNproc'  => $aNproc,
                'mNproc'  => $mNproc,
                'lNproc'  => $lNproc,
                'aIO'     => $aIO,
                'mIO'  => $mIO,
                'lIO'  => $lIO,
                'memf'    => $memf,
                'mepf'    => $mepf,
                'pmepf'   => $pmepf,
                'nprocf'  => $nprocf,
                'apcpu'   => $apcpu,
                'mpcpu'   => $mpcpu,
                'aMemStr' => RoundMemB($aMem),
                'mMemStr' => RoundMemB($mMem),
                'lMemStr' => RoundMemB($lMem),
                'eapcpu'  => isError( $apcpu > 75 ),
                'empcpu'  => isError( $mpcpu > 90 ),
                'eaep'    => isError( ( $lep > 0 ) && ( $aep * 2 > $lep ) ),
                'emep'    => isError( ( $lep > 0 ) && ( $mep > $lep * .9 ) ),
                'eaMem'   => isError( ( $lMem > 0 ) && ( $aMem * 2 > $lMem ) ),
                'emMem'   => isError( ( $lMem > 0 ) && ( $mMem > $lMem * .9 ) ),
                'ememf'   => isError( $memf > 0 ),
                'emepf'   => isError( $mepf > 0 )
            }
        );
    }
    return \@RSD;
}

sub api2 {
    my ($func) = @_;
    my %API = (
       'getUsage' => {
           'func'   => 'api2_getUsage',
            'engine' => 'hasharray',
        },
       'getUsage2' => {
           'func'   => 'api2_getUsage2',
            'engine' => 'hasharray',
        },
        getSummary  => {
            'func'  => 'api2_getSummary',
            'engine'=> 'hasharray',
        },
        processPHPVersionSelect  => {
            'func'  => 'api2_processPHPVersionSelect',
            'engine'=> 'hasharray',
        },
        showPHPExtTable  => {
            'func'  => 'api2_showPHPExtTable',
            'engine'=> 'hasharray',
        },
        showPHPOptTable  => {
            'func'  => 'api2_showPHPOptTable',
            'engine'=> 'hasharray',
        },
    );
    return \%{ $API{$func} };
}

sub api2_processPHPVersionSelect {
    # returns alternatives state for a user
    # and applies alternatives selection if
    # the form is submitted
    unless (check_license()) {
        return [ { errmsg => 'PHP Selector disabled, please contact administrator' } ]
    }
    # get parameters from calling side
    my %opts = @_;

    # get current user name
    my $user = scalar getpwuid($<);
    
    # default command string to display summary
    my $param_string = qq#/usr/bin/cl-selector --summary php --user $user|#;

    # if form has been submitted, we first apply settings
    # and then show summary
    if ( $opts{lveversion} and $opts{lveversion} ne 'none' ) {
        $param_string = qq#/usr/bin/cl-selector --select php --user $user --version=$opts{lveversion} --return-summary|#;
    }

    # open pipe for command and grab output to string
    open my $p, $param_string;
    my $raw_string = do { local $/; <$p> };
    close $p;

    my @options;
    # if returned string starts with 'ERROR', something wrong has happened
    # Strip 'ERROR' word and send error string to be displayed
    if ( index( $raw_string, 'ERROR' ) >= 0 ) {
        $raw_string =~ s/^.*ERROR:(.*?)\n/$1/s;
        return [ { errmsg => $raw_string } ];
    }
	elsif ( $raw_string =~ s/WARN:(.*?)\n// ) {
		push @options, { warnmsg => $1 };
	}
    # if no error then parse returned string into array of arrays
    my @raw_options = map { [ split ] } split( /\n/, $raw_string );
    
    # data is returned in the following form for entry: 'version e d s'
    # 'e' stands for enabled, 'd' for default and 's' for selected.
    # opposite value is marked with a hyphen ('-')
    # process already structured data taking into account
    # if entry is marked as 'selected' or 'disabled'

    for my $i ( @raw_options ) {
        my $ver = $i->[0];
        my $state = $i->[1] ne 'e'
            ? q#disabled#
            : $i->[3] eq 's'
                ? q#selected#
                : ''
                ;

        # and save'em in array
        push @options, { state=>$state, ver=>$ver };
    }
    return \@options;
}

sub api2_showPHPExtTable {
    # returns extension table
    unless (check_license()) {
        return []
    }
    # get current user
    my $user = scalar getpwuid($<);

    # open pipe for command and grab output to string
    open my $p, qq#/usr/bin/cl-selector --list-extensions=php --user=$user --all|#;
    my $raw_string = do { local $/; <$p> };

    # if returned string starts with 'ERROR', something wrong has happened
    # We are not about to show anything if error
    if ( index( $raw_string, 'ERROR' ) == 0 or index( $raw_string, 'WARN' ) == 0) {
        return [];
    }
    my $data = eval `/usr/bin/cl-phpextdesc --perl`;
    $data = [] unless ref $data eq 'ARRAY';
    my %data_desc = map { $_->{title} => $_->{description} } @{ $data };

    # if no error then parse returned string into array of hashes
    my @ext = 
        sort { $a->{title} cmp $b->{title} }
            map {
                my ( $s, $n ) = split;
                { title => $n,
                  state => ( $s eq '+' ? 1 : 0 ),
                  desc  => ( $data_desc{$n} || q{} ) };
            } split( /\n/, $raw_string );
    return \@ext;
}

sub api2_showPHPOptTable {
    # returns options table
    unless (check_license()) {
        return []
    }
    # get current user
    my $user = scalar getpwuid($<);

    # open pipe for command and grab output to string
    my $data = eval `/usr/bin/piniset --user=$user --perl`;
    return [] unless ref $data eq 'ARRAY';
    return $data;
}

sub LVEInfo_PHPExtDesc {
    # print titles for extensions (for javascript)
    open my $p,qq#/usr/bin/cl-phpextdesc --json|#;
    print do { local $/; <$p>; };
    close $p;
}

sub get_usage_info {
    return api2_getUsage( 'period' => '1d' );
}

sub check_license {
    my @params = ('/usr/bin/cldetect', '--check-license');
    my $result = Cpanel::SafeRun::Errors::saferunallerrors(@params);
    chomp $result;
    if ($result eq 'OK') {
        return 1;
    }
    return 0;
}

sub LVEInfo_print_usage_overview {

    unless (check_license()) {
        print '<strong>Resource Usage monitoring disabled, please contact administrator</strong>';
        return;
    }

    my $usage = get_usage_info();

    unless (@$usage) {
        print '<h3>There has been no activity on your site within the past 24 hours</h3><br/>';
        print q#<p><a href="resourceusagedetails.html">[Details]</a></p>#;
        return;
    }

    my $cpu_limited_flag = 0;
    my $memf_total       = 0;
    my $mepf_total       = 0;
    my $cpu_max          = 0;
    my $mem_max          = 0;
    my $lmem_max         = 0;
    my $ep_max           = 0;
    my $lep_max          = 0;
    my $pmem_max         = 0;
    my $lpmem_max        = 0;
    my $pmemf_total      = 0;
    my $nproc_max        = 0;
    my $lnproc_max       = 0;
    my $nprocf_total     = 0;
    my $iousage_max      = 0;
    my $iolimit_max      = 0;

    foreach my $hash (@$usage) {
        $cpu_limited_flag = 1 if ( $hash->{'mcpu'} >= $hash->{'lcpu'} );
        $memf_total   += $hash->{'memf'};
        $mepf_total   += $hash->{'mepf'};
        $pmemf_total  += $hash->{'pmepf'};
        $nprocf_total += $hash->{'nprocf'};
        $cpu_max    = $hash->{'mpcpu' } if ( $hash->{'mpcpu' } > $cpu_max );
        $mem_max    = $hash->{'mMem'  } if ( $hash->{'mMem'  } > $mem_max );
        $lmem_max   = $hash->{'lMem'  } if ( $hash->{'lMem'  } > $lmem_max );
        $ep_max     = $hash->{'mep'   } if ( $hash->{'mep'   } > $ep_max );
        $lep_max    = $hash->{'lep'   } if ( $hash->{'lep'   } > $lep_max );
        $lpmem_max  = $hash->{'lPMem' } if ( $hash->{'lPMem' } > $lpmem_max );
        $pmem_max   = $hash->{'mPMem' } if ( $hash->{'mPMem' } > $pmem_max );
        $nproc_max  = $hash->{'mNproc'} if ( $hash->{'mNproc'} > $nproc_max );
        $lnproc_max = $hash->{'lNproc'} if ( $hash->{'lNproc'} > $lnproc_max );
        $iousage_max  = $hash->{'mIO'} if ( $hash->{'mIO'} > $iousage_max );
        $iolimit_max = $hash->{'lIO'} if ( $hash->{'lIO'} > $iolimit_max );
    }

    if ( ($cpu_limited_flag) || ( $memf_total > 0 ) || ( $mepf_total > 0 ) ) {
        print '<h3>Your site has been limited within the past 24 hours</h3>';

        if ($cpu_limited_flag) {
            print '<p>CPU resources were limited for your site';
        }

        if ( $memf_total > 0 ) {
            print '<p>Virtual memory resources were limited for your site';
        }

        if ( $pmemf_total > 0 ) {
            print '<p>Physical memory resources were limited for your site';
        }

        if ( $mepf_total > 0 ) {
            print '<p>You have reached entry processes (number of simultaneously running php and cgi scripts, ', 'as well as cron jobs and shell sessions) limit <b>', $mepf_total, '</b> times';
        }
        if ( $nprocf_total > 0 ) {
            print '<p>You have reached processes number limit <b>', $nprocf_total, '</b> times';
        }

    }
    else {
        print '<h3>Your site had no issues in the past 24 hours</h3>';
    }

    if (   ( ( !$cpu_limited_flag ) && ( $cpu_max > 90 ) )
        || ( ( $memf_total == 0 ) && ( $lmem_max > 0 ) && ( $mem_max > $lmem_max * 0.9 ) )
        || ( ( $pmemf_total == 0 ) && ( $lpmem_max > 0 ) && ( $pmem_max > $lpmem_max * 0.9 ) )
        || ( ( $mepf_total == 0 ) && ( $lep_max > 0 )  && ( $ep_max > $lep_max * 0.9 ) )
        || ( ( $nprocf_total == 0 ) && ( $lnproc_max > 0 )  && ( $nproc_max > $lnproc_max * 0.9 ) )
        || ( ( $iolimit_max > 0 )  && ( $ioisage_max > $iolimit_max * 0.9 ) ) ) {
        print '<p><h3>Your site might hit resource limits soon</h3>';

        if ( ( !$cpu_limited_flag ) && ( $cpu_max > 90 ) ) {
            print '<p>Your CPU usage was at ', $cpu_max, '% out of 100%';
        }

        if ( ( $memf_total == 0 ) && ( $lmem_max > 0 ) && ( $mem_max > $lmem_max * 0.9 ) ) {
            print '<p>Your Virtual Memory usage was at ', RoundMemB($mem_max), ' out of ', RoundMemB($lmem_max);
        }

        if ( ( $pmemf_total == 0 ) && ( $lpmem_max > 0 ) && ( $pmem_max > $lpmem_max * 0.9 ) ) {
            print '<p>Your Physical Memory usage was at ', RoundMemB($pmem_max), ' out of ', RoundMemB($lpmem_max);
        }

        if ( ( $mepf_total == 0 ) && ( $lep_max > 0 ) && ( $ep_max > $lep_max * 0.9 ) ) {
            print '<p>You had ', $ep_max, ' entry processes (number of simultaneously running php and cgi scripts, ', 'as well as cron jobs and shell sessions) out of ', $lep_max, ' max entry processes allowed';
        }

        if ( ( $nprocf_total == 0 ) && ( $lnproc_max > 0 ) && ( $nproc_max > $lnproc_max * 0.9 ) ) {
            print '<p>You had ', $nproc_max, ' processes out of ', $lnproc_max, ' max processes allowed';
        }

        if ( ( $iolimit_max > 0 ) && ( $iousage_max > $iolimit_max * 0.9 ) ) {
            print '<p>You had ', $iousage_max, ' I/O usage out of ', $iolimit_max, ' max I/O usage allowed';
        }
    }
    print q#<p><a href="resourceusagedetails.html">[Details]</a></p>#;
}

# Function converts bytes to kilobytes, megabytes or gigabytes (if needed)
# and appends appropriate suffix (K, M, G)
# Parameter $_[1] = memory in bytes
sub RoundMemB {
    my $mem = shift;

    if ( $mem >= ( 1024 * 1024 * 1024 ) ) {
        return sprintf( "%.1fG", $mem / ( 1024 * 1024 * 1024 ) );
    }

    if ( $mem >= 1024 * 1024 ) {
        return sprintf( "%.1fM", $mem / ( 1024 * 1024 ) );
    }

    if ( $mem >= 1024 ) {
        return sprintf( "%.0fK", $mem / 1024 );
    }

    return "$mem";
}

# Function converts kilobytes to megabytes or gigabytes (if needed)
# and appends appropriate suffix (K, M, G)
# Parameter $_[1] = memory in kilobytes
sub RoundMemK {
    my $mem = shift;

    if ( $mem >= ( 1024 * 1024 ) ) {
        return sprintf( "%.1fG", $mem / ( 1024 * 1024 ) );
    }

    if ( $mem >= 1024 ) {
        return sprintf( "%.1fM", $mem / 1024 );
    }

    return "$mem" . 'K';
}

sub cpanel_11_40_or_above {
    # 11.40.0 converted to integer
    my $cpanel_11_40 = 753664;

    # get cpanel version
    my $cmd = '/usr/local/cpanel/cpanel -V';
    open my $h, "$cmd|";
    my ($raw_version) = <$h> =~ /([\d\.]+)/;
    
    # got unexpected and exit as if version less than 1.40.0
    return 0 unless $raw_version;
    close $h;

    # split version to separate digits and make integer from them
    my @digits = (split /\./, $raw_version);
    my $int_version = $digits[0] << 16 | $digits[1] << 12 | $digits[2] << 6;
    return $int_version >= $cpanel_11_40 ? 1 : 0;
}

sub is_file_usage_enabled {
    return 0 unless cpanel_11_40_or_above();

    my $cpanel_config = '/var/cpanel/cpanel.config';
    return 0 unless -e $cpanel_config;

    # precompile regexp for efficiency
    my $file_usage_pattern = qr/file_usage=1/;
    
    my $enabled = 0;
    open my $f, '<', $cpanel_config or return 0;
    while ( my $line = <$f> ) {
        if ( $line =~ /$file_usage_pattern/ ) {
            $enabled = 1;
            last;
        }
    }
    close $f;
    return $enabled;
}

sub k_to_m {
    my ( $val ) = @_;
    my $rv = $val / 1024;
    my $fmt = ($rv == int $rv) ? "%d" : "%.1f";
    return sprintf $fmt, $rv;
}

1;
