#!/bin/sh
eval 'if [ -x /usr/local/cpanel/3rdparty/bin/perl ]; then exec /usr/local/cpanel/3rdparty/bin/perl -x -- $0 ${1+"$@"}; else exec /usr/bin/perl -x -- $0 ${1+"$@"};fi'
if 0;
#!/usr/bin/perl

use strict;
use Getopt::Std;

my %opts = ();
getopts( 'iud',\%opts );

unless ( $> == 0 ) {
    warn "The program should be started under root privileges only!\n";
    exit(0);
}

my $theme_location = '/usr/local/cpanel/base/frontend';

my @data = (
    {
        src => q#/usr/share/l.v.e-manager/cpanel/cgi/CloudLinux.cgi#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/cgi/CloudLinux.cgi#,
        bcp => 1,
        check_orig => 1,
    },
    {    
        src => q#/usr/share/l.v.e-manager/cpanel/resourceusage/LVEInfo.pm#,
        dst => q#/usr/local/cpanel/Cpanel/LVEInfo.pm#,
        bcp => 1,
        check_orig => 0,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/resourceusage/lvechart.cgi#,
        dst => q#/usr/local/cpanel/base/3rdparty/cloudlinux/lvechart.cgi#,
    },

# substitution for lveversion.cgi which is obsolete
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/cl-selector.cgi#,
        dst => q#/usr/local/cpanel/base/3rdparty/cloudlinux/cl-selector.cgi#,
    },

# javascript moved from lveselect.html to separate files
    {
        src => q#/usr/share/l.v.e-manager/commons/js/user-extension-selector.js#,
        dst => q#/usr/local/cpanel/base/3rdparty/cloudlinux/user-extension-selector.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/commons/js/user-option-selector.js#,
        dst => q#/usr/local/cpanel/base/3rdparty/cloudlinux/user-option-selector.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/commons/js/cl-message-handler.js#,
        dst => q#/usr/local/cpanel/base/3rdparty/cloudlinux/cl-message-handler.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/jsonparse.js#,
        dst => q#/usr/local/cpanel/base/3rdparty/cloudlinux/jsonparse.js#,
    },

# javacript moved from CloudLinux.cgi to separate files
    {
        src => q#/usr/share/l.v.e-manager/commons/js/extension-selector.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/extension-selector.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/commons/js/cl-message-handler.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/cl-message-handler.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/cgi-lib/cloudlinux/cl-selector-toggle.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/cl-selector-toggle.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/cgi-lib/cloudlinux/lvetablefilter.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/lvetablefilter.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/cgi-lib/cloudlinux/lvetablesort.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/lvetablesort.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/cgi-lib/cloudlinux/lvetop.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/lvetop.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/cgi-lib/cloudlinux/packagehide.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/packagehide.js#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/jsonparse.js#,
        dst => q#/usr/local/cpanel/whostmgr/docroot/3rdparty/cloudlinux/jsonparse.js#,
    },
);

my @theme_data = (
    # resource usage
    {
        src => q#/usr/share/l.v.e-manager/cpanel/resourceusage/resourceusage.html#,
        dst => q#stats/resourceusage.html#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/resourceusage/resourceusagedetails.html#,
        dst => q#stats/resourceusagedetails.html#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/resourceusage/lveusage.tmpl#,
        dst => q#stats/lveusage.tmpl#,
    },
    
    # PHP selector
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/extension-selector.tmpl#,
        dst => q#lveversion/extension-selector.tmpl#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/option-selector.tmpl#,
        dst => q#lveversion/option-selector.tmpl#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/extension-selector.html#,
        dst => q#lveversion/extension-selector.html#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/option-selector.html#,
        dst => q#lveversion/option-selector.html#,
    },
    {
        src => q#/usr/share/l.v.e-manager/cpanel/lveversion/version-selector.tmpl#,
        dst => q#lveversion/version-selector.tmpl#,
    },
);


if ( $opts{i} or $opts{u} ) {
    for my $i ( @data ) {
        if ( -e $i->{src} ) {
            if ( $i->{bcp} ) {
                make_backup( $i->{dst}, 0, $i->{check_orig} ? 1 : 0 );
            }
            else {
                unlink $i->{dst} if -e $i->{dst};
            }
            check_dir( $i->{dst} );
            system( "/bin/cp", $i->{src}, $i->{dst} );
        }
    }
    for my $theme ( @{ get_themes_list() } ) {
        for my $i ( @theme_data ) {
            if ( -e $i->{src} ) {
                my $path = $theme . '/' . $i->{dst};
                unlink $path if -e $path;
                check_dir($path);
                system( "/bin/cp", $i->{src}, $path );
            }
        }
        if ( $opts{u} ) {
            unpatch_index($theme);
        }
        patch_index($theme);
        check_dynamicui($theme);
    }
    install_locales();
    update_rvskin();
    #process_obsoletes();
}
elsif ( $opts{d} ) {
    for my $i ( @data ) {
        unlink $i->{dst} if -e $i->{dst};
        my $bcp = $i->{dst} . '.saved';
        my $orig = $i->{dst} . '.orig';
        if ( -e $orig ) {
            system( "/bin/mv", $orig, $i->{dst} );
            unlink $bcp if -e $bcp;
        }
        elsif ( -e $bcp ) {
            system( "/bin/mv", $bcp, $i->{dst} );
        }
    }
    for my $theme ( @{ get_themes_list() } ) {
        for my $item ( @theme_data ) {
            my $path = $theme . '/' . $item->{dst};
            unlink $path if -e $path;
        }
        unpatch_index($theme);
    }
    remove_locales();
    update_rvskin();
}

sub check_dynamicui {
    my ( $path ) = @_;
    my $dynamicui_x3_path = q#/usr/local/cpanel/base/frontend/x3/dynamicui#;
    my $dynamicui_x3_conf = q#/usr/local/cpanel/base/frontend/x3/dynamicui.conf#;
    my $d_path = join( q{/}, $path, 'dynamicui' );
    unless ( -e join( q{/}, $path, 'dynamicui.conf' ) ) {
        system( '/bin/cp', $dynamicui_x3_conf, $path );
    }
    return unless -d $dynamicui_x3_path;
    opendir my $dyn_x3, $dynamicui_x3_path or return;
    mkdir $d_path unless -d $d_path;
    for my $d ( readdir $dyn_x3 ) {
        next if $d eq '.' or $d eq '..';
        next unless $d =~ /(?:enduserlve|lvephpsel)/i;
        my $item_path = join( q{/}, $dynamicui_x3_path, $d );
        my @params = ( '/bin/cp', $item_path, $d_path );
        system( @params ) unless -e join( q{/}, $d_path, $d );
    }
    closedir $dyn_x3;
}

sub update_rvskin {
    system( '/root/rvadmin/autoupdatewhmaddon.pl' ) if -x '/root/rvadmin/autoupdatewhmaddon.pl';
}

sub process_obsoletes {
    my @deletes = (
        '/usr/local/cpanel/base/3rdparty/lveversion.cgi',
        '/usr/local/cpanel/base/frontend/x3/lveversion/lveselect.tmpl',
        '/usr/local/cpanel/base/3rdparty/lvechart.cgi',
        '/usr/local/cpanel/base/3rdparty/lvechart.cgi.saved',
    );
    for my $i ( @deletes ) {
        unlink $i if -e $i;
    }
}

sub get_themes_list {
    my @dirs;
    return [] unless -d $theme_location;
    opendir my $dir, $theme_location or return [];
    for my $d ( readdir $dir ) {
        next if $d eq '.' or $d eq '..';
        my $path = $theme_location . '/' . $d;
        next if -l $path;
        next unless -d $path;
        push @dirs, $path;
    }
    closedir $dir;
    return \@dirs;
}

sub check_dir {
    my ( $path ) = @_;
    my $dirpath = substr( $path, 0, rindex( $path, '/' ) );
    system("/bin/mkdir", "-p", $dirpath) unless -d $dirpath;
}

sub make_backup {
    my ( $path, $to_copy, $check_orig ) = @_;
    return unless -e $path;
    my $bcp;   
    if ( $check_orig and detect_orig( $path ) ) {
        $bcp = $path . '.orig';
    }
    else {
        $bcp = $path . '.saved';
        unlink $bcp if -e $bcp;
    }
    if ( $to_copy ) {
        system( "/bin/cp", $path, $bcp );
    }
    else {
        system( "/bin/mv", $path, $bcp );
    }
}

sub detect_orig {
    my ( $path ) = @_;
    return 0 unless -e $path;
    open my $f, '<', $path or return 0;
    my ( $str_count, $str_limit ) = ( 0, 30 );
    while ( my $line = <$f> ) {
        last if $str_count > $str_limit;
        if ( index( $line, '#WHMADDON:lvemanager:LVE Manager' ) >= 0 ) {
            # This is the cPanel original file
            return 1;
        }
        $str_count++;
    }
    close $f;
    return 0;
}

sub patch_index {
    my ( $theme_path ) = @_;
    my $path = $theme_path . '/branding/index.html';
    my ( $cc, $qc, $hc, $ps, $qs ) # common_condition, quota_contition, hide_condition, param_string, param_quota_string
        = (
        q# && !$CONF{'lve_hideuserstat'} && !$CONF{'lve_showinodeusage'}>#,
        q# && !$CONF{'lve_hideuserstat'} && $CONF{'lve_showinodeusage'}>#,
        q# && $CONF{'lve_hideuserstat'}>#,
        q#lvecpu|lvemem|lvepmem|lvemep|lveproc|lveiop|#,
        q#lvecpu|lvemem|lvepmem|lvemep|lveproc|lveiop|lvequota|#,
    );
    my $search_pattern = qr/
                        ( \n \s* \<cpanelif\s\$hascloudlinux ) \>
                            (?= \s+ <\?\s?cp\sStatsBar::stat )
                                ( .+? display= ) .*?
                                ( diskusage .+? \<\/cpanelif\> \s*? )
                            \n
                    /sx;
    return unless -e $path;
    open my $i, '<', $path or return;
    my $data = do { local $/; <$i> };
    close $i;
    $data =~ s/$search_pattern/$1$cc$2$ps$3$1$qc$2$qs$3$1$hc$2$3\n/;
    open my $o, '>', $path or return;
    print $o $data;
    close $o;
}

sub unpatch_index {
    my ( $theme_path ) = @_;
    my $path = $theme_path . '/branding/index.html';
    my $saved = $theme_path . '/branding/index.saved';
    my $orig = $theme_path . '/branding/index.orig';
    my $ps = q#lvecpu|lvemem|lvemep|#; # param_string
    my $search_pattern = qr/
                            ( \n \s* \<cpanelif\s\$hascloudlinux ) .+? \>
                                (?= \s+ <\?\s?cp\sStatsBar::stat )
                                    ( .+? display= ) .*?
                                    ( diskusage .+? \<\/cpanelif\> \s*? )
                            (?:\1 .+? \> \2 .*? \3)?
                            \1 .+? \> \2 .*? \3 \n
                        /sx;
    unlink $saved if -e $saved;
    unlink $orig if -e $orig;
    return unless -e $path;
    open my $i, '<', $path or return;
    my $data = do { local $/; <$i> };
    close $i;
    $data =~ s/$search_pattern/$1>$2$ps$3\n/;
    open my $o, '>', $path or return;
    print $o $data;
    close $o;
}

sub install_locales {
    my $src = q#/usr/share/l.v.e-manager/cpanel/locales#;
    my $dst = q#/var/cpanel/locale.local#;
    if ( -d $src ) {
        chdir $src;
        for my $file ( glob('*.yaml') ) {
            system('/bin/cp', '-f', $file, $dst)
        }
        system('/usr/local/cpanel/bin/build_locale_databases');
    }
}

sub remove_locales {
    my $path = q#/var/cpanel/locale.local#;
    if ( -d $path ) {
        chdir $path;
        for my $file( glob('*.yaml') ) {
            unlink $file;
        }
        system('/usr/local/cpanel/bin/build_locale_databases');
    }
}
