<?php

/*
     * Title: LVEManager Module.
     * Version: 1.1.0 (04/Nov/2011)
     * Author: Krishtopin Anton. AKrishtopin@cloudlinux.com
     * Site: http://CloudLinux.com
*/

class LVE {
    private $ProcLVEVersion;
    private $PanelUsers;
    private $PanelPackages;
    
    public function __construct() {
        $result = self::CommandExecute('/usr/bin/sudo /usr/sbin/lvectl lve-version');
        $result = explode(PHP_EOL,$result);
        $this->ProcLVEVersion = $result[0];
        self::GetPanelUsers();
        self::GetPanelPackages();
        unset($result);
    }
    
    // Execute Shell Command
    private function CommandExecute($command,$result = true) {
        if ($result) {
            ob_start();
            passthru($command);
            $return = ob_get_contents();
            ob_end_clean();
            return $return;
//            return explode(PHP_EOL,$return);
        } else {
            passthru($command);
            return true;
        }
    }

    // Get users domains   
    private function GetUsersDomains() {
        $data = self::CommandExecute('/usr/bin/sudo /usr/local/directadmin/plugins/new_lvemanager/admin/GetDomains');
        $data = explode(PHP_EOL,$data);
        unset($data[count($data)-1]);
        $result = array();
        foreach ($data as $item) {
            $tmp = explode(' ',$item);
            $result[$tmp[0]] = $tmp[1];
        }
        return $result;
    }
    
    // Simply Get Users from control panel
    private function GetPanelUsers() {
        $result = self::CommandExecute('/usr/bin/sudo /usr/sbin/lvectl paneluserslimits');
        
        if ($this->ProcLVEVersion == 6) {
            $counter = 8;
        } else {
            $counter = 6;
        }
        
        while (strpos($result,'  ') !== false) {
            $result = str_replace('  ',' ',$result);
        }
        $tmp = explode(' ',trim($result));
        for ($i=0; $i<=$counter; $i++) {
            $blank[$i] = trim($tmp[$i]);
        }
        foreach ($tmp as $key => $value) {
            if (($key % ($counter+1)) == 0)
                $uid = $tmp[$key];
            if ($uid != 'ID')
                $data[$uid][$blank[($key%($counter+1))]] = str_replace(PHP_EOL,'',$value);
        }

        $domains = self::GetUsersDomains();

        foreach ($data as $key => $item) {
            if ($key != 0) {
                $data[$key]['UserName'] = str_replace(PHP_EOL,'',self::CommandExecute('getent passwd '.$key.' | cut -f 1 -d ":"'));
                $data[$key]['Domain'] = $domains[$data[$key]['UserName']];
            } else {
                $data[$key]['UserName'] = '-';
                $data[$key]['Domain'] = '-';
                $data[$key]['ID'] = 'DEFAULT';
            }
        }
        foreach ($data as $key => $value) {
            $data[$key]['IOPRIO'] = $data[$key]['IO'];
        }
        
        $this->PanelUsers = $data;
        unset($data,$tmp,$counter,$uid,$blank,$result);
    }
    
    // Simply Get Packages from control panel
    private function GetPanelPackages() {
        $result = self::CommandExecute('/usr/bin/sudo /usr/sbin/lvectl package-list');
        
        if ($this->ProcLVEVersion == 6) {
            $counter = 7;
        } else {
            $counter = 5;
        }
        
        while (strpos($result,'  ') !== false) {
            $result = str_replace('  ',' ',$result);
        }
        $tmp = explode(' ',trim($result));
        for ($i=0; $i<=$counter; $i++) {
            $blank[$i] = trim($tmp[$i]);
        }
        foreach ($tmp as $key => $value) {
            if (($key % ($counter+1)) == 0)
                $uid = $tmp[$key];
            if ($uid != 'ID')
                $data[$uid][$blank[($key%($counter+1))]] = $value;
        }
        
        foreach ($data as $key => $value) {
            $data[$key]['IOPRIO'] = $data[$key]['IO'];
        }
        
        $this->PanelPackages = $data;
        unset($data,$tmp,$counter,$uid,$blank,$result);
    }
    
    // return ProcLVE version
    public function ReturnProcLVEVerion() {
        return $this->ProcLVEVersion;
    }
    
    // return panel users and limits
    public function ReturnPanelUsers() {
        return $this->PanelUsers;
    }
    
    // return panel Packages
    public function ReturnPanelPackages(){
        return $this->PanelPackages;
    }
    
    // delete user settings
    public function SetUserToDefault($UID) {
        self::CommandExecute('/usr/bin/sudo /usr/sbin/lvectl delete '.$UID,false);
    }
    
    //delete package settings
    public function SetPackageToDefault($Package) {
        self::CommandExecute('/usr/bin/sudo /usr/sbin/lvectl package-delete "'.$Package.'"',false);
    }
    
    // Setup LVE for user
    public function SetupLVE($UID,$params) {
        self::CommandExecute('/usr/bin/sudo /usr/sbin/lvectl set '.$UID.$params,false);
    }
    
    // Setup Package
    public function SetupPackage($Package,$params) {
        echo '/usr/bin/sudo /usr/sbin/lvectl package-set "'.$Package.'"'.$params;
        self::CommandExecute('/usr/bin/sudo /usr/sbin/lvectl package-set "'.$Package.'"'.$params,false);
    }
    
    public function GetStatData($params) {
        $result = self::CommandExecute('/usr/sbin/lveinfo '.$params);
        $result = explode(PHP_EOL,$result);
        $tmp = $result[0];
        unset($result[0]);
        unset($result[count($result)]);
        $tmp = explode(',',$tmp);
        $i = 0;
        $keys = array();
        foreach ($tmp as $value) {
            $keys[$i] = $value;
            $i++;
        }
        
        $return = array();
        $j = 0;
        foreach ($result as $item) {
            $tmp = explode(',',$item);
            $i = 0;
            foreach ($tmp as $value) {
                $return[$j][$keys[$i]] = $value;
                $i++;
            }
            $j++;
        }
        return $return;
    }
}

$LVE = new LVE();