<?php
    require_once('/usr/local/directadmin/plugins/new_lvemanager/check_utils.php');
    $description = array ();
    $content = file('/etc/cl.selector/phpextdesc.txt');

    foreach ($content as $line) {
        $tmp = explode('=',$line);
        $description[trim($tmp[0])] = trim($tmp[1]);
    }

    function GetDescription($field_name) {
        global $description;
        if (array_key_exists($field_name, $description)) {
            return $description[$field_name];
        } else {
            return 'no info';
        }
    }

    class Selector {
        private $SelectorStatus = 0;
        private $SelectorConfig = '/usr/local/directadmin/plugins/phpselector/plugin.conf';
        private $DefaultVersion;
        private $SelectorVersions = array();
        private $SelectorExtensions = array();
        public $TableRows;
        // User Variables
        private $UserName;
        private $UserVersion;
        private $UserSelectorVersions = array();
        private $UserSelectorExtensions = array();
        private $UserSelectorOptions = array();
        public $Error = null;


        public function __construct() {
            $content = trim(file_get_contents($this->SelectorConfig));
            if (strstr($content,'active=yes')) {
                $this->SelectorStatus = 1;
            }
        }

        // Execute Shell Command
        private function CommandExecute($command,$result = true,$lines_output = false) {
            if ($result) {
                ob_start();
                passthru($command);
                $return = ob_get_contents();
                ob_end_clean();
                if ($lines_output) {
                    $return = explode(PHP_EOL,$return);
                    unset($return[count($return)-1]);
                }
                return $return;
            } else {
                passthru($command);
                return true;
            }
        }

        // Returns Json description of extensions
        public function ReturnExtDescriptionJson() {
            return self::CommandExecute('/usr/bin/cl-phpextdesc --json',true);
        }

        // Return Selector Status (1 - enabled, 0 - disabled)
        public function GetSelectorStatus() {
            return $this->SelectorStatus;
        }

        // Change Selector Status (1 - enabled, 0 - disabled)
        public function ChangeSelectorStatus() {
            $content = trim(file_get_contents($this->SelectorConfig));
            if ($this->SelectorStatus == 0) {
                $content = str_replace('active=no','active=yes',$content);
                if (file_put_contents($this->SelectorConfig,$content)): return true; else: return false; endif;
            } else {
                $content = str_replace('active=yes','active=no',$content);
                if (file_put_contents($this->SelectorConfig,$content)): return true; else: return false; endif;
            }
        }

        // Get default version & supported versions
        public function GetSelectorVersions() {
            $result = self::CommandExecute('/usr/bin/sudo /usr/bin/cl-selector --summary=php',true,true);
            foreach ($result as $item) {
                $tmp = explode(' ',$item);
                $this->SelectorVersions[$tmp[0]]['version'] = trim($tmp[0]);
                $this->SelectorVersions[$tmp[0]]['enabled'] = trim($tmp[1]);
                $this->SelectorVersions[$tmp[0]]['default'] = trim($tmp[2]);
                if (trim($tmp[2]) == 'd') {
                    $this->DefaultVersion = trim($tmp[0]);
                }
            }
        }

        // Returns Default PHP version
        public function GetDefaultVersion() {
            return $this->DefaultVersion;
        }

        // Returns Versions
        public function GetVersions() {
            return $this->SelectorVersions;
        }

        // Change Default PHP Selector version
        public function ChangeDefaultVersion($version) {
            $version = self::getUserVersion($version);
            self::CommandExecute('/usr/bin/sudo /usr/bin/cl-selector --interpreter=php --version='.trim($version),false);
        }

        // Enable/Disable php version
        public function ChangeVersionStatus($version, $action) {
            $action_command = ($action != 'enabled') ? " --set-enabled={$version}" : " --set-disabled={$version}";
            self::CommandExecute('/usr/bin/sudo /usr/bin/cl-selector --interpreter=php'.$action_command);
        }

        // Get extensions for php version
        public function GetSelectorExtensions($version = '') {
            $version = check_version($version);
            if ($version == '') {
                $version = self::GetDefaultVersion();
            }
            $result = self::CommandExecute('/usr/bin/sudo /usr/bin/cl-selector --list-extensions=php --version='.$version,true,true);
            $i = 0;
            foreach ($result as $line) {
                $tmp = explode(' ',$line);
                $this->SelectorExtensions[$i]['name']=$tmp[1];
                $this->SelectorExtensions[$i]['status']=$tmp[0];
                $i++;
            }
            $this->TableRows = ceil(count($this->SelectorExtensions)/5);
        }

        // returns selector Extensions
        public function ReturnSelectorExt() {
            return $this->SelectorExtensions;
        }

        // Update Extensions
        public function UpdateExtensionsList($version, $extensions) {
            $result = self::CommandExecute('/usr/bin/sudo /usr/bin/cl-selector --interpreter=php --version='.$version.' --update-default-extensions="'.$extensions.'"',true,true);
            return $result;
        }

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////// User Interface Functions ////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        // Get User name & PHP Versions
        public function UserGetData($NewVersion = '') {
            $this->UserName = check_user($_SERVER['USER']);
            $NewVersion = check_version($NewVersion);
            if ($NewVersion != '') {
                self::CommandExecute('/usr/bin/cl-selector --select=php --version='.$NewVersion.' --user='.$this->UserName);
            }
            $result = self::CommandExecute('/usr/bin/cl-selector --summary=php --user '.$this->UserName,true,true);
            if (strstr($result[0],'ERROR')) {
                $this->Error = trim($result[0]);
            } else {
                foreach ($result as $item) {
                    $tmp = explode(' ',$item);
                    $this->UserSelectorVersions[$tmp[0]]['version'] = trim($tmp[0]);
                    $this->UserSelectorVersions[$tmp[0]]['enabled'] = trim($tmp[1]);
                    $this->UserSelectorVersions[$tmp[0]]['default'] = trim($tmp[2]);
                    $this->UserSelectorVersions[$tmp[0]]['selected'] = trim($tmp[3]);
                    if (trim($tmp[3]) == 's') {
                        $this->UserVersion = trim($tmp[0]);
                    }
                }
            }
        }

        // Just a wrapper around version checks
        public function getUserVersion($version) {
            $version = check_version($version);
            if ($version == '') {
                $version = self::UserReturnPHPVersion($this->UserName);
            }
            return $version;
        }

        // Return User PHP version
        public function UserReturnPHPVersion($username='') {
            if (isset($this->UserVersion)) {
                return $this->UserVersion;
            }
            if ($username != '') {
                $commandString = '/usr/bin/cl-selector --current=php --user='.username;
                $result = self::CommandExecute($commandString, true, false);
                $data = preg_split("/\s+/", $result);
                return $data[0];
            }
        }

        // Return User Selector Versions
        public function UserReturnSelectorVersions() {
            return $this->UserSelectorVersions;
        }

        // Get User Extensions for PHP version (Selected by default)
        public function UserGetExtensions($version = '') {
            $version = self::getUserVersion($version);
            $result = self::CommandExecute('/usr/bin/cl-selector --list-extensions=php --version='.$version.' --user '.$this->UserName.' --all',true,true);
            $i = 0;
            foreach ($result as $line) {
                $tmp = explode(' ',$line);
                $this->UserSelectorExtensions[$i]['name']=$tmp[1];
                $this->UserSelectorExtensions[$i]['status']=$tmp[0];
                $i++;
            }
            $this->TableRows = ceil(count($this->UserSelectorExtensions)/5);
        }

        // returns User Extensions
        public function UserReturnSelectorExt() {
            return $this->UserSelectorExtensions;
        }

        // Set Defaults to User
        public function UserSetDefaults($version) {
            $version = self::getUserVersion($version);
            self::CommandExecute('/usr/bin/cl-selector --set-default-extensions=php --user='.$this->UserName.' --version='.$version.'');
        }

        // Save Extensions
        public function UserSaveExtList($version, $ExtList) {
            $result = self::CommandExecute('/usr/bin/cl-selector --interpreter=php --update='.$ExtList.' --user='.$this->UserName.' --version='.$version.' --with-reload --backup',true,true);
            return $result;
        }

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// User SELECTOR options /////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        // Get User Selector Options for PHP version (Selected by default)
        public function UserGetOptions($version = '', $json = false) {
            $version = self::getUserVersion($version);
            $this->UserSelectorOptions = array();
            if ($json) {
                $result = self::CommandExecute('/usr/bin/piniset --user '.$this->UserName.' --json',true);
                return $result;
            } else {
                $result = self::CommandExecute('/usr/bin/piniset --user='.$this->UserName,true,true);
                $tmp = array();
                foreach ($result as $line) {
                    if (strstr($line,'TITLE')) {
                        if (!empty($tmp)) {
                            $this->UserSelectorOptions[] = $tmp;
                        }
                        $tmp = array();
                        $res = explode(':',$line,2);
                        $tmp[trim($res[0])] = trim($res[1]);
                    } elseif (trim($line) != '') {
                        $res = explode(':',$line,2);
                        $tmp[trim($res[0])] = trim($res[1]);
                    }
                }
                if (!empty($tmp)) {
                    $this->UserSelectorOptions[] = $tmp;
                }
            }
        }

        // Return User Selector Options
        public function UserReturnSelectorOptions() {
            return $this->UserSelectorOptions;
        }

        // Save user options
        public function UserSetOptions($version, $options) {
            $version = self::getUserVersion($version);
//            echo '/usr/bin/piniset --version='.$version.' --user '.$this->UserName.' --replace='.$options;
            $result = self::CommandExecute('/usr/bin/piniset --version='.$version.' --user '.$this->UserName.' --replace='.$options.' --base64',true,true);
        }
    }
