#!/usr/bin/python

import os
import pprint
import sys
import getopt
import shutil
import stat
import glob
import grp
import pwd

#import lve_diagnostic
#from lve_diagnostic import *
import cldetectlib as detect
import copy_directory
from copy_directory import *
import exec_command
from exec_command import *
import mysql_lib
from mysql_lib import *
from pwd import getpwnam
from grp import getgrnam


SOURCE_PATH = "/usr/share/l.v.e-manager/"
ROOT_PLESK_DIR = "/usr/local/psa/admin/"
ROOT_CPANEL_DIR = "/usr/local/cpanel/whostmgr/docroot/"
ROOT_IWORX_DIR = "/usr/local/interworx/plugins/"
ROOT_ISPMGR_DIR = "/usr/local/ispmgr/"
ROOT_DA_DIR = "/usr/local/directadmin/plugins/"
CPANEL_ENDUSER_DIR = "/usr/local/cpanel/base/frontend/x3/"

def recursive_file_permissions(path,uid=-1,gid=-1):
    for item in glob.glob(path+'/*'):
        if os.path.isdir(item):
            recursive_file_permissions(os.path.join(path,item),uid,gid)
        else:
            try:
                os.chown(os.path.join(path,item),uid,gid)
            except:
                print('File permissions on {0} not updated due to error.')


def touch(fname):
    try:
        os.utime(fname, None)
    except:
        open(fname, 'a').close()


def getItem(txt1, txt2, op):
    try:
        i1 = int(txt1)
    except ValueError:
        i1 = -1
    try:
        i2 = int(txt2)
    except ValueError:
        i2 = -1
    if i1 == -1 or i2 == -1:
        if op == 0:
            return txt1>txt2
        else:
            return txt1<txt2
    else:
        if op == 0:
            return i1>i2
        else:
            return i1<i2


#Compare version of types xx.xx.xxx... and yyy.yy.yy.y..
#if xxx and yyy is numbers, than comapre as numbers
#else - comapre as strings
def verCompare (base, test):
    base = base.split(".")
    test = test.split(".")
    if(len(base)>len(test)):
        ln = len(test)
    else:
        ln = len(base)
    for i in range(ln):
        if getItem(base[i],test[i],0):
            return 1
        if getItem(base[i],test[i],1):
            return -1
    if len(base)==len(test):
        return 0
    elif len(base)>len(test):
        return 1
    else:
        return 0


def usage():
    print ''
    print 'Use following syntax to manage CageFS istall utility:'
    print sys.argv[0]+" [OPTIONS]"
    print 'Options:'
    print " -i | --install     : install lvemanager plugin"
    print " -d | --delete      : delete  lvemanager plugin"
    print " -u | --update      : update  lvemanager plugin"


def update_sudoers(user):
    def exit_with_error(message, status=1):
        sys.stderr.write("%s\n" % (str(message,)))
        sys.exit(status)
    try:
        from clsudo import (
            Clsudo,
            NoSuchUser,
            UnableToReadFile,
            UnableToWriteFile)
        sudo = Clsudo()
        sudo.add_user(user)
    except ImportError:
        exit_with_error(
        "Cannot import Clsudo. Check if python-cllib package installed")
    except (NoSuchUser, UnableToReadFile, UnableToWriteFile), e:
        exit_with_error(e)
    except Exception, e:
        exit_with_error(e)


def install_plugin(tp):
    # install only. rewrite all existing file
    update_php_config()
    if cp.name == "Plesk" and verCompare (cp.version, "10") >= 0:
        update_sudoers('psaadm')
        cpdir = CopyDirectory(SOURCE_PATH+"plesk", ROOT_PLESK_DIR)
        cpdir.process()
        cpdir = CopyDirectory(SOURCE_PATH+"commons", ROOT_PLESK_DIR + 'htdocs/modules/plesk-lvemanager')
        cpdir.process()
        if os.path.exists(SOURCE_PATH+"utils/install-plesk-plugin.sh"):
            exec_command(SOURCE_PATH+"utils/install-plesk-plugin.sh")
        if os.path.exists(SOURCE_PATH+"utils/plesk-cgi.sh"):
            if not os.path.exists("/usr/share/l.v.e-manager/plesk/script/"):
                os.makedirs("/usr/share/l.v.e-manager/plesk/script/")
            shutil.copyfile(SOURCE_PATH+"utils/plesk-cgi.sh", "/usr/share/l.v.e-manager/plesk/script/plesk-cgi.sh")
            if os.path.exists(SOURCE_PATH+"utils/installsupergrp.sh"):
                exec_command(SOURCE_PATH+"utils/installsupergrp.sh psaadm")
    elif cp.name == "cPanel":

        cpdir = CopyDirectory(SOURCE_PATH+"cpanel/images", ROOT_CPANEL_DIR+"images/")
        cpdir.process()

        cpdir = CopyDirectory(SOURCE_PATH+"cpanel/images_enduser/tablesort", CPANEL_ENDUSER_DIR+"images/")
        cpdir.process()

        if os.path.exists(SOURCE_PATH+"cpanel/hooks/cpanel-install-hooks"):
            exec_command(SOURCE_PATH+"cpanel/hooks/cpanel-install-hooks")
        if os.path.exists(SOURCE_PATH+"cpanel/resourceusage/resourceusage.plugin"):
            exec_command("/usr/local/cpanel/bin/register_cpanelplugin "+SOURCE_PATH+"cpanel/resourceusage/resourceusage.plugin")
        if os.path.exists(SOURCE_PATH+"cpanel/lveversion/selectphpversion.plugin"):
            exec_command("/usr/local/cpanel/bin/register_cpanelplugin "+SOURCE_PATH+"cpanel/lveversion/selectphpversion.plugin")
        if os.path.exists(SOURCE_PATH+"cpanel/utils/install_module.pl"):
            action_key = ( ( tp == "install" and '-i' ) or '-u' )
            exec_command("perl "+SOURCE_PATH+"cpanel/utils/install_module.pl " + action_key)

    elif cp.name == "InterWorx":
        cpdir = CopyDirectory(SOURCE_PATH+"interworx/", ROOT_IWORX_DIR, "*", "*", 0)
        cpdir.process()
        exec_command("chmod -R 755 "+ROOT_IWORX_DIR+"lvemanager")
        update_sudoers('iworx')
        if os.path.exists(SOURCE_PATH+"utils/installsupergrp.sh"):
            exec_command(SOURCE_PATH+"utils/installsupergrp.sh iworx")
    elif cp.name == "ISPManager":
        res1 = exec_command("/usr/sbin/lvectl --lve-version")[0].strip()
        try:
            i = int(res1)
        except ValueError:
            i = 0
        if i<6:
            cpdir = CopyDirectory(SOURCE_PATH+"ispmanager/liblve0.8", ROOT_ISPMGR_DIR, "*", "*", 0, copy_file_mode=True)
            cpdir.process()
        else:
            cpdir = CopyDirectory(SOURCE_PATH+"ispmanager/liblve1.1", ROOT_ISPMGR_DIR, "*", "*", 0, copy_file_mode=True)
            cpdir.process()
        exec_command("/usr/bin/killall ispmgr")
        exec_command(SOURCE_PATH+"utils/install-isp-plugin.sh")
    elif cp.name == "DirectAdmin":
        cpdir = CopyDirectory(SOURCE_PATH+"directadmin/", ROOT_DA_DIR,"*","*",0)
        cpdir.process()
        cpdir = CopyDirectory(SOURCE_PATH+"commons/js", ROOT_DA_DIR + 'new_lvemanager/admin/phpselector',"*","*",0)
        cpdir.process()
        exec_command("chown -R diradmin:diradmin "+ROOT_DA_DIR+"new_lvemanager")
        exec_command("chown -R diradmin:diradmin "+ROOT_DA_DIR+"phpselector")
        exec_command("chmod -R 755 "+ROOT_DA_DIR+"new_lvemanager")
        exec_command("chmod -R 755 "+ROOT_DA_DIR+"phpselector")
        exec_command("chmod 777 "+ROOT_DA_DIR+"phpselector/plugin.conf")
        exec_command(SOURCE_PATH+"utils/install-da-plugin.sh")
        admin_user = detect.detect_DA_admin()
        update_sudoers(admin_user)
    else:
        print "Current panel unsupported. Panel name: "+cp.name+" version: "+cp.version

def delete_plugin():
    # remove installed files
    if cp.name == "Plesk" and verCompare (cp.version, "10") >= 0:
        if os.path.exists(SOURCE_PATH+"utils/delete-plesk-plugin.sh"):
            exec_command(SOURCE_PATH+"utils/delete-plesk-plugin.sh")
        if os.path.exists(ROOT_PLESK_DIR+"htdocs/images/modules/plesk-lvemanager"):
            shutil.rmtree(ROOT_PLESK_DIR+"htdocs/images/modules/plesk-lvemanager")
        if os.path.exists(ROOT_PLESK_DIR+"htdocs/modules/plesk-lvemanager"):
            shutil.rmtree(ROOT_PLESK_DIR+"htdocs/modules/plesk-lvemanager") 
        if os.path.exists(ROOT_PLESK_DIR+"plib/modules/plesk-lvemanager"):
            shutil.rmtree(ROOT_PLESK_DIR+"plib/modules/plesk-lvemanager")
        if os.path.exists(ROOT_PLESK_DIR+"plib/templates/modules/plesk-lvemanager"):
            shutil.rmtree(ROOT_PLESK_DIR+"plib/templates/modules/plesk-lvemanager")
        if os.path.exists(ROOT_PLESK_DIR+"htdocs/chart.php"):
            os.remove(ROOT_PLESK_DIR+"htdocs/chart.php")
        if os.path.exists(ROOT_PLESK_DIR+"htdocs/lve.php"):
            os.remove(ROOT_PLESK_DIR+"htdocs/lve.php")
    elif cp.name == "cPanel":
        exec_command(SOURCE_PATH+"cpanel/hooks/cpanel-delete-hooks")
        if os.path.exists(SOURCE_PATH+"cpanel/utils/install_module.pl"):
            exec_command("perl "+SOURCE_PATH+"cpanel/utils/install_module.pl -d")
        if os.path.exists(ROOT_CPANEL_DIR+"images/lve"):
            shutil.rmtree(ROOT_CPANEL_DIR+"images/lve")
        if os.path.exists(ROOT_CPANEL_DIR+"images/CloudLinux_logo.png"):
            os.remove(ROOT_CPANEL_DIR+"images/CloudLinux_logo.png")
        if os.path.exists(CPANEL_ENDUSER_DIR+"branding/lvephpsel.jpg"):
            os.remove(CPANEL_ENDUSER_DIR+"branding/lvephpsel.jpg")
        if os.path.exists(SOURCE_PATH+"cpanel/resourceusage/resourceusage.plugin"):
            exec_command("/usr/local/cpanel/bin/unregister_cpanelplugin "+SOURCE_PATH+"cpanel/resourceusage/resourceusage.plugin")
        if os.path.exists(SOURCE_PATH+"cpanel/lveversion/selectphpversion.plugin"):
            exec_command("/usr/local/cpanel/bin/unregister_cpanelplugin "+SOURCE_PATH+"cpanel/lveversion/selectphpversion.plugin")

    elif cp.name == "InterWorx":
        if os.path.exists(ROOT_IWORX_DIR+"lvemanager"):
            shutil.rmtree(ROOT_IWORX_DIR+"lvemanager")
    elif cp.name == "ISPManager":
        if os.path.exists(ROOT_ISPMGR_DIR+"addon/lvemanager"):
            shutil.rmtree(ROOT_ISPMGR_DIR+"addon/lvemanager")
        if (os.path.exists("/usr/local/ispmgr/addon/lvemanager.php")):
            os.remove("/usr/local/ispmgr/addon/lvemanager.php")
        if (os.path.exists("/usr/local/ispmgr/addon/LVEPackage.php")):
            os.remove("/usr/local/ispmgr/addon/LVEPackage.php")
        if (os.path.exists("/usr/local/ispmgr/addon/LVEPackageUser.php")):
            os.remove("/usr/local/ispmgr/addon/LVEPackageUser.php")

        if (os.path.exists("/usr/local/ispmgr/etc/ispmgr_mod_lvemanager.xml")):
            os.remove("/usr/local/ispmgr/etc/ispmgr_mod_lvemanager.xml")
        if (os.path.exists("/usr/local/ispmgr/etc/ispmgr_mod_lvepreset.xml")):
            os.remove("/usr/local/ispmgr/etc/ispmgr_mod_lvepreset.xml")

        if (os.path.exists("/usr/local/ispmgr/skins/mobile/mb-lve.png")):
            os.remove("/usr/local/ispmgr/skins/mobile/mb-lve.png")
        if (os.path.exists("/usr/local/ispmgr/skins/mobile/mb-resource.png")):
            os.remove("/usr/local/ispmgr/skins/mobile/mb-resource.png")
        if (os.path.exists("/usr/local/ispmgr/skins/mobile/m-lve.png")):
            os.remove("/usr/local/ispmgr/skins/mobile/m-lve.png")
        if (os.path.exists("/usr/local/ispmgr/skins/mobile/m-resource.png")):
            os.remove("/usr/local/ispmgr/skins/mobile/m-resource.png")
        if (os.path.exists("/usr/local/ispmgr/skins/mobile/refresh_auto.png")):
            os.remove("/usr/local/ispmgr/skins/mobile/refresh_auto.png")
        if (os.path.exists("/usr/local/ispmgr/skins/mobile/refresh.png")):
            os.remove("/usr/local/ispmgr/skins/mobile/refresh.png")

        if (os.path.exists("/usr/local/ispmgr/sirius/mobile/mb-lve.png")):
            os.remove("/usr/local/ispmgr/sirius/mobile/mb-lve.png")
        if (os.path.exists("/usr/local/ispmgr/sirius/mobile/mb-resource.png")):
            os.remove("/usr/local/ispmgr/sirius/mobile/mb-resource.png")
        if (os.path.exists("/usr/local/ispmgr/sirius/mobile/m-lve.png")):
            os.remove("/usr/local/ispmgr/sirius/mobile/m-lve.png")
        if (os.path.exists("/usr/local/ispmgr/sirius/mobile/m-resource.png")):
            os.remove("/usr/local/ispmgr/sirius/mobile/m-resource.png")
        if (os.path.exists("/usr/local/ispmgr/sirius/mobile/refresh_auto.png")):
            os.remove("/usr/local/ispmgr/sirius/mobile/refresh_auto.png")
        if (os.path.exists("/usr/local/ispmgr/sirius/mobile/refresh.png")):
            os.remove("/usr/local/ispmgr/sirius/mobile/refresh.png")

        exec_command("/usr/bin/killall ispmgr")
        exec_command(SOURCE_PATH+"utils/delete-isp-plugin.sh")

    elif cp.name == "DirectAdmin":
        if os.path.exists(ROOT_DA_DIR+"new_lvemanager"):
            shutil.rmtree(ROOT_DA_DIR+"new_lvemanager")
        if os.path.exists(ROOT_DA_DIR+"phpselector"):
            shutil.rmtree(ROOT_DA_DIR+"phpselector")
    else:
        print "Current panel unsupported. Panel name: "+cp.name+" version: "+cp.version

def update_php_config():
    """
    Adds defaults for error_reporting option
    (to avoid empty value) and tries to keep
    all user changes untouched
    """
    config_path = '/etc/cl.selector/php.conf'
    data = {}
    data_index = []
    if not os.path.exists(config_path):
        return
    fd = open(config_path,'r')
    directive = None
    for line in fd:
        if '=' not in line:
            continue
        if line.startswith('Directive'):
            directive = line[ line.find('=')+1 : ].strip()
            data_index.append(directive)
            data[directive] = {}
            if not 'data' in data[directive]:
                data[directive]['data'] = {}
            if not 'index' in data[directive]:
                data[directive]['index'] = []
            continue
        if not directive:
            continue
        ( key, value ) = map( ( lambda x: x.strip() ), line.split('=', 1) )
        data[directive]['index'].append(key.lower())
        data[directive]['data'][ key.lower() ] = value
    fd.close()
    if 'error_reporting' not in data:
        return
    if 'default' in data['error_reporting']['data'] and data['error_reporting']['data']['default'] != "":
        return
    if 'default' not in data['error_reporting']['data']:
        data['error_reporting']['index'].insert(0,'default')
    data['error_reporting']['data']['default'] = 'E_ALL & -E_NOTICE'
    fd = open( config_path, 'w' )
    for directive in data_index:
        fd.write( "Directive = %s\n" % directive )
        for item in data[directive]['index']:
            fd.write( "%s = %s\n" % ( item.capitalize().ljust(9), data[directive]['data'][item]) )
        fd.write("\n")
    fd.close()


def cp_supported():
    if cp.name == "Plesk" and verCompare (cp.version, "10") >= 0:
        return True
    if cp.name in ("cPanel", "InterWorx", "ISPManager", "DirectAdmin"):
        return True
    return False


class ControlPanel:
    name = ''
    varsion = ''

    def __init__(self, name, version):
        self.name = name
        self.version = version

detect.getCP()
cp = ControlPanel(detect.CP_NAME, detect.CP_VERSION)

try:
    opts, args = getopt.getopt(sys.argv[1:], "hidu", ["help", "install", "delete", "update"])
except getopt.GetoptError, err:
    # print help information and exit:
    print str(err) # will print something like "option -a not recognized"
    usage()
    sys.exit(2)

for o, a in opts:
    if o in ("-h", "--help"):
        usage()
        sys.exit()
    elif o in ("-i", "--install"):
        install_plugin("install")
    elif o in ("-d", "--delete"):
        delete_plugin()
    elif o in ("-u", "--update"):
        install_plugin("update")
    else:
        usage()
        sys.exit(2)


