<?php
/* Copyright Cloud Linux Inc 2010-2011 All Rights Reserved                                                                                                   
 *                                                                                                                                                           
 * Licensed under CLOUD LINUX LICENSE AGREEMENT                                                                                                              
 * http://cloudlinux.com/docs/LICENSE.TXT                                                                                                                    
 */
require_once("modules/plesk-lvemanager/checkData.php");

if ( ! function_exists('json_encode') ) {
    function json_encode($data) {
        switch (gettype($data)) {
            case 'NULL':
                return 'null';
            case 'boolean':
                return $data ? 'true' : 'false';
            case 'integer':
                return (int) $data;
            case 'double':
            case 'float':
                return (float) $data;
            case 'string':
                $result = '';
                $string_length = strlen($data);
                for ( $i=0; $i<$string_length; $i++ ) {
                    $char_code = ord( $data{$i} );
                    if ( $char_code == 0x08 ) {
                        $result .= '\b';
                    }
                    elseif ( $char_code == 0x09 ) {
                        $ascii .= '\t';
                    }
                    elseif ( $char_code == 0x0A ) {
                        $result .= '\n';
                    }
                    elseif ( $char_code == 0x0C ) {
                        $result .= '\f';
                    }
                    elseif ( $char_code == 0x0D ) {
                        $result .= '\r';
                    }
                    elseif ( $char_code == 0x22 or $char_code == 0x2F or $char_code == 0x5C ) {
                        $result .= '\\' . $data{$i};
                    }
                    elseif ( $char_code >= 0x20 and $char_code <= 0x7F ) {
                        $result .= $data{$i};
                    }
                    elseif ( ( $char_code & 0xE0 ) == 0xC0 ) {
                        $c = pack( 'C*', $char_code, ord($data{$i+1}) );
                        $i += 1;
                        $result .= sprintf( '\u%04s', bin2hex( covert_to_utf16($c) ) );
                    }
                    elseif ( ( $char_code & 0xF0) == 0xE0 ) {
                        $c = pack( 'C*', $char_code, ord($data{$i+1}), ord($data{$i+2}) );
                        $i += 2;
                        $result .= sprintf( '\u%04s', bin2hex( covert_to_utf16($c) ) );
                    }
                    elseif ( ( $char_code & 0xF8) == 0xF0 ) {
                        $c = pack( 'C*', $char_code, ord($data{$i+1}), ord($data{$i+2}), ord($data{$i+3}) );
                        $i += 3;
                        $result .= sprintf( '\u%04s', bin2hex( covert_to_utf16($c) ) );
                    }
                    elseif ( ( $char_code & 0xFC) == 0xF8 ) {
                        $c = pack( 'C*', $char_code, ord($data{$i+1}), ord($data{$i+2}), ord($data{$i+3}), ord($data{$i+4}) );
                        $i += 4;
                        $result .= sprintf( '\u%04s', bin2hex( covert_to_utf16($c) ) );
                    }
                    elseif ( ( $char_code & 0xFE) == 0xFC ) {
                        $c = pack( 'C*', $char_code, ord($data{$i+1}), ord($data{$i+2}), ord($data{$i+3}), ord($data{$i+4}), ord($data{$i+5}) );
                        $i += 5;
                        $result .= sprintf( '\u%04s', bin2hex( covert_to_utf16($c) ) );
                    }
                }
                return '"' . $result . '"';
            case 'array':
                if ( is_array($data) and count($data) and array_keys($data) !== range( 0, count($data)-1 ) ) {
                    $params = array_map( 'encode_mapper', array_keys($data), array_values($data) );
                    return '{' . join(',', $params) . '}';
                }
                $items = array_map('json_encode', $data);
                return '[' . join(',', $items) . ']';
            case 'object':
                $items = get_object_vars($data);
                $params = array_map( 'encode_mapper', array_keys($items), array_values($items) );
                return '{' . join(',', $params) . '}';
            default:
                return null;
        }
    }
    function encode_mapper($key, $val) {
        return json_encode(strval($key)) . ':' . json_encode($val);
    }
    function covert_to_utf16($data){
        if(function_exists('mb_convert_encoding')) {
            return mb_convert_encoding($data, 'UTF-16', 'UTF-8');
        }
        switch( strlen($data) ) {
            case 1:
                return $data;
            case 2:
                return chr(0x07 & (ord($data{0}) >> 2)) . chr((0xC0 & (ord($data{0}) << 6)) | (0x3F & ord($data{1})));
            case 3:
                return chr((0xF0 & (ord($utf8{0}) << 4)) | (0x0F & (ord($utf8{1}) >> 2))) . chr((0xC0 & (ord($utf8{1}) << 6)) | (0x7F & ord($utf8{2})));
        }
        return '';
    }
}

function stripWarnsOrErrors($str){
    if ( $pos = strpos( $str, "ERROR:") !== false ) {
        $epos = strpos( $str, "\n", $pos );
        $messages[] = substr( $str, $pos+5, $epos-6);
        return array("status"=>"ERROR","message"=>$messages);
    }
    while ( ( $pos = strpos( $str, "WARN:" ) ) !== false ) {
        $epos = strpos( $str, "\n", $pos );
        $msg = substr( $str, $pos + 5, $epos - $pos - 5 );
        $messages[] =  $msg;
        $str = str_replace( substr ( $str, $pos, $epos - $pos + 1 ), "", $str );
    }
    if ( isset($messages) ) {
        return array( "status"=>"WARN", "message"=>$messages, "data"=>$str );
    }
    return array("status"=>"OK","data"=>$str);
}

if ( isset($_POST['action'] ) ) {
    $action = $_POST['action'];
    $dispatchTable = array(
        'extlist'       => getExtensionList,
        'extreset'      => getExtensionList,
        'extbuiltin'    => getBuiltinList,
        'extsave'       => saveExtensionSet,
        'versionsave'   => versionSave,
        'versiontoggle' => versionToggle,
    );
    if ( array_key_exists( $action, $dispatchTable ) ) {
        $dispatchTable[$action]();
    }
    else {
        $dispatchTable['extlist']();
    }
}

function getExtensionList($default_version=null) {
    $version = checkVersion($_POST['version']);
    $version = isset($default_version) ? $default_version : $version;
    ob_start();
    passthru("/usr/bin/cl-selector --list-extensions=php --version=" . $version);
    $data = ob_get_contents();
    ob_end_clean();
    $structuredData = stripWarnsOrErrors($data);
    if ( array_key_exists( 'data', $structuredData ) ) {
        $eData = explode( "\n", rtrim($structuredData['data']) );
        foreach ( $eData as $ext ) {
            $eItem = explode( " ", rtrim($ext) );
            switch ( $eItem[0] ) {
                case '+':
                    $state = 1;
                    break;
                case '-':
                    $state = 0;
                    break;
                case '~':
                    $state = -1;
                    break;
                default:
                    $state = 0;
            }
            $extData[] = array('title' => $eItem[1], 'status' => $state);
        }
        $structuredData['data'] = $extData;
    }
    $result = array();
    if ( array_key_exists( 'status', $structuredData ) ) {
        $result['status'] = $structuredData['status'];
    }
    if ( array_key_exists( 'message', $structuredData ) ) {
        $result['message'] = $structuredData['message'];
    }
    if ( array_key_exists( 'data', $structuredData ) ) {
        $result['data'] = $structuredData['data'];
    }
    header('Content-Type: application/json');
    print json_encode($result);
}

function getBuiltinList() {
    getExtensionList('native');
}

function saveExtensionSet() {
    $version = checkVersion($_POST['version']);
    $extset = checkDataSet($_POST['extset']);
    ob_start();
    passthru( "sudo /usr/bin/cl-selector --interpreter=php --version=".$version." --update-default-extensions=".$extset );
    $data = ob_get_contents();
    ob_end_clean();
    $extlist = array();
    foreach ( explode( ',', $extset ) as $ext ) {
        $extlist[$ext] = 1;
    }
    if ( $data ) { # we have messages
        $structuredData = stripWarnsOrErrors($data);
        if ( ( $pos = strpos( $data, "ERROR:") ) !== false ) {
            $epos = strpos( $data, "\n", $pos );
            $result = array( 'status' => 'ERROR', 'message' => substr( $data, $pos+5, $epos-6) );
        }
        elseif ( ( $pos = strpos( $data, "WARN:") ) !== false ) {
            $pattern = '/WARN:(\S+)\s(\S+).*?(?:\((.*)\))?\.?$/';
            
            foreach ( explode( "\n", $data ) as $line ) {
                $matches = array();
                preg_match($pattern, $line, $matches);

                // if extension was skipped (for conflict), remove it
                if ( $matches[2] === 'skipped' and array_key_exists( $matches[1], $extlist ) ) {
                    $extlist[ $matches[1] ] = 0;
                }
                // if extension was enabled (for dependency), add it
                elseif ( $matches[2] === 'enabled' ) {
                    $extlist[ $matches[1] ] = 1;
                }

                // if extension was left (for dependency), add it
                elseif ( $matches[2] === 'left' ) {
                    $extlist[ $matches[1] ] = 1;
                }
            }
            $data = preg_replace( "/WARN:/", "", $data );
            $outlist = array();
            foreach ( $extlist as $key => $value ) {
                $outlist[] = array('title' => $key, 'status' => $value );
            }
            $result = array( 'status' => 'WARN', 'message' => explode( "\n", $data ), 'data'=>$outlist );
        }
    }
    else {
        $result = array('status' => 'OK' );
    }
    header('Content-Type: application/json');
    print json_encode( $result );
}

function versionSave() {
    $version = checkVersion($_POST['version']);
    ob_start();
    passthru("sudo /usr/bin/cl-selector --interpreter=php --version=" . $version);
    $data = ob_get_contents();
    ob_end_clean();
    header('Content-Type: application/json');
    if ( ! $data ) {
        print json_encode( array('status' => 'OK') );
    }
    else {
        print json_encode( stripWarnsOrErrors($data) );
    }
}

function versionToggle() {
    $version = checkVersion($_POST['version']);
    $state = checkState($_POST['state']);
    $action_arg = ( ( $state == 'disabled' ) ? "--set-enabled=" . $version : "--set-disabled=" . $version );
    ob_start();
    passthru( "sudo /usr/bin/cl-selector --interpreter=php " . $action_arg );
    $data = ob_get_contents();
    ob_end_clean();
    header('Content-Type: application/json');
    if ( ! $data ) {
        print json_encode( array('status' => 'OK') );
    }
    else {
        print json_encode( stripWarnsOrErrors($data) );
    }
}
?>
