<?php
/* Copyright Cloud Linux Inc 2010-2011 All Rights Reserved                                                                                                   
 *                                                                                                                                                           
 * Licensed under CLOUD LINUX LICENSE AGREEMENT                                                                                                              
 * http://cloudlinux.com/docs/LICENSE.TXT                                                                                                                    
*/ 

require_once('modules/pm.php');

require_once ('modules/plesk-lvemanager/home.php');
require_once ('modules/plesk-lvemanager/default.php');
require_once ('modules/plesk-lvemanager/packages.php');
require_once ('modules/plesk-lvemanager/packagesedit.php');
require_once ('modules/plesk-lvemanager/accounts.php');
require_once ('modules/plesk-lvemanager/stats.php');
require_once ('modules/plesk-lvemanager/edit.php');
require_once ('modules/plesk-lvemanager/history.php');
require_once ('modules/plesk-lvemanager/selector.php');


class ModuleForm extends pm_Form
{

    private $_activeTab = 'home';
    private $_allowedTabs = array('home', 'packages', 'accounts', 'stats', 'selector');
    private $_action;
    private $_isAdmin;

    public function setActiveTab($tabId)
    {
	if (in_array($tabId, $this->_allowedTabs)) {
	    $this->_activeTab = $tabId;
        }
    }

    public function isAdmin()
    {
        if (!isset($this->_isAdmin)) {
            $session = new pm_Session();
            $client = $session->getClient();
            $this->_isAdmin = $client->isAdmin();
        }
        return $this->_isAdmin;
    }

    public function setAction($actId)
    {
        $this->_action = $actId;
    }

    public function define()
    {
	if(trim($this->_action)=="")
    	    $this->addTemplates(array(
        	'main' => 'main.tpl',
        	'form' => 'form.tpl',
        	'lve-module' => "modules/plesk-lvemanager/{$this->_activeTab}.tpl",
    	    ));
         else 
	    $this->addTemplates(array(
        	'main' => 'main.tpl',
        	'form' => 'form.tpl',
        	'lve-module' => "modules/plesk-lvemanager/{$this->_action}.tpl",
    	    ));

        parent::define();
    }


    public function assign()
    {
        $tabs = new ScreenTabs($this->_activeTab);
        $tabs->addScreenTab('home', 'Home', '?tab=home');
	$tabs->addScreenTab('accounts', 'Settings', '?tab=accounts');
	$tabs->addScreenTab('stats', 'Statistics', '?tab=stats');
	$tabs->addScreenTab('packages', 'Packages', '?tab=packages');
	$tabs->addScreenTab('selector', 'PHP Selector', '?tab=selector');

        $vars = array(
            'SCREEN_TABS' => $tabs->get(),
        );

	switch($this->_activeTab){
	    case 'accounts':
		switch($this->_action){
		case "edit":
		    $hm = new EditPage($this);
		    $vars=$hm->getResult($vars);
		break;
		case "history":
		    $hm = new HistoryPage($this);
		    $vars=$hm->getResult($vars);
		break;
		default:
		    $hm = new AccountPage($this);
		    $vars=$hm->getResult($vars);
		}
	    break;
	    case 'stats':
		$hm = new StatsPage($this);
		$vars=$hm->getResult($vars);
	    break;
	    case 'packages':
		switch($this->_action){
		case "packagesedit":
		    $hm = new PackagesEditPage($this);
		    $vars=$hm->getResult($vars);
		break;
		default:
		    $hm = new PackagesPage($this);
		    $vars=$hm->getResult($vars);
		}
		
	    break;
	    case 'selector':
		$hm = new SelectorPage($this);
		$vars=$hm->getResult($vars);
	    break;
	    default:
		$hm = new HomePage($this);
		$vars=$hm->getResult($vars);
	}

        $this->addVars($vars);
        parent::assign();
    }

    public function parse()
    {
        //parent::parse('FORM', 'lve-module');
        //parent::parse('BODY', 'form');
	parent::parse('BODY', 'lve-module');
        parent::parse('DOCUMENT', 'main');
    }
}

