<?php
/* Copyright Cloud Linux Inc 2010-2011 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * http://cloudlinux.com/docs/LICENSE.TXT
*/

function stripWarnsOrErrors($str){
    $messages = array();
    if ( $pos = strpos( $str, "ERROR:") !== false ) {
        $epos = strpos( $str, "\n", $pos );
        $messages[] = substr( $str, $pos-1, $epos+1);
        return array("messages"=>$messages);
    }
    while ( ( $pos = strpos( $str, "WARN:" ) ) !== false ) {
        $epos = strpos( $str, "\n", $pos );
        $msg = substr( $str, $pos+5, $epos+5);
        $messages[] =  $msg;
        $str = str_replace( substr($str,$pos,$epos+1), "", $str );
    }
    return array("messages"=>$messages,"data"=>$str);
}

function getUserSelectorData($user){
    ob_start();
    passthru("/usr/bin/cl-selector --summary=php --user=" . $user);
    $data = ob_get_contents();
    ob_end_clean();
    $data = stripWarnsOrErrors($data);
    if ( ! array_key_exists( 'data', $data ) ) {
        if (  array_key_exists( 'messages', $data ) ) {
            return $data;
        }
    }
    $vData = explode( "\n", rtrim($data['data']) );
    foreach ( $vData as $item ) {
        $vItem = explode( " ", rtrim($item) );
        $state = $vItem[1] == 'e' ? 1 : 0;
        if ( $vItem[2] == "d" ) { $state = 2; }
        if ( $vItem[3] == "s" ) { $state = 3; }
        $versions[ $vItem[0] ] = $state;
    }

    if ( array_key_exists( 'messages', $data ) and count( $data['messages'] ) != 0 ) {
        return array('messages'=>$data['messages'],'data'=>$versions );
    }
    return array('data'=>$versions);
}

function getUserExtensionsData($version, $user, $extDesc){
    ob_start();
    passthru("/usr/bin/cl-selector --list-extensions=php --version=" . $version . " --user=" . $user . " --all");
    $data = ob_get_contents();
    ob_end_clean();
    $data = stripWarnsOrErrors($data);
    if ( ! array_key_exists( 'data', $data ) ) {
        if (  array_key_exists( 'messages', $data ) ) {
            return $data;
        }
    }
    $eData = explode( "\n", rtrim($data['data']) );
    
    // Strangely but instead of empty array we get
    // array of one empty string element. That's why
    // such a crutch
    if ( count( $eData) == 1 and $eData[0] == "" ) {
        unset( $eData[0] );
    }
    
    foreach ( $eData as $ext ) {
        $eItem = explode( " ", rtrim($ext) );
        $iDesc = array_key_exists( $eItem[1], $extDesc ) ? $extDesc[ $eItem[1] ] : "no info";
        switch ( $eItem[0] ) {
            case '+':
                $ctrl = '<td title="' . $iDesc . '" style="padding:.4em 0 0 0;">' . $eItem[1] . '</td>';
                $ctrl .= '<td title="' . $iDesc . '" style="padding:0 1.6em 0 .8em;"><input type="checkbox" name="cbx_' . $eItem[1] . '" checked="checked"/></td>';
                break;
            case '-':
                $ctrl = '<td title="' . $iDesc . '" style="padding:.4em 0 0 0;">' . $eItem[1] . '</td>';
                $ctrl .= '<td title="' . $iDesc . '" style="padding:0 1.6em 0 .8em;"><input type="checkbox" name="cbx_' . $eItem[1] . '"/></td>';
                break;
            case '~':
                $ctrl = '<td title="' . $iDesc . '" style="padding:.4em 0 0 0;font-style:italic">' . $eItem[1] . '</td>';
                $ctrl .= '<td title="' . $iDesc . '" style="padding:0 1.6em 0 .8em;"><input type="checkbox" name="cbx_' . $eItem[1] . '" checked="checked" disabled="disabled"/></td>';
                break;
            default:
                $ctrl = '<td title="' . $iDesc . '" style="padding:.4em 0 0 0">' . $eItem[1] . '</td>';
                $ctrl .= '<td title="' . $iDesc . '" style="padding:0 1.6em 0 .8em;"><input type="checkbox" name="cbx_' . $eItem[1] . '"/></td>';
        }
        $extData[] = $ctrl;
    }
    if ( array_key_exists( 'messages', $data ) ) {
        return array('messages'=>$data['messages'],'data'=>$extData );
    }
    return array('data'=>$extData);
}

function getUserExtensionDescriptions(){
    ob_start();
    passthru('/usr/bin/cl-phpextdesc');
    $data = split( "\n", rtrim( ob_get_contents() ) );
    ob_end_clean();
    foreach ( $data as $line ) {
        $pair = split( "=", $line );
        $desc[ $pair[0] ] = $pair[1];
    }
    return $desc;
}

function getUserOptionsData($username){
    ob_start();
    passthru('/usr/bin/piniset --user='.$username.' --csv');
    $data = ob_get_contents();
    ob_end_clean();
    $optData = "";
    foreach ( explode( "\n", rtrim($data) ) as $row ) {
        $array = str_getcsv($row);
        if ( $array[0] == 'status' and $array[1] == 'ERROR' ) {
            return array('status'=>'ERROR','message'=>$array[3]);
        }
        $row = array();
        for ( $i = 1; $i != count($array); $i += 2 ) {
            $row[ $array[$i] ] = $array[ $i+1 ];
        }
        $optData .= '<tr style="border-bottom:1px solid #D8D8D8;"title="'.$row['comment'].'">';
        $optData .= '<td style="padding:0 32px 0 12px;font-weight:bold;">'.$array[0].'</td>';
        if ( array_key_exists('value', $row ) ) {
            $optData .= '<td style="height:24px;" id="option-value_'.$array[0].'">';
            $optData .= '<a href="#" style="color:#008C23;" class="option-link_'.$array[0].'">'.$row['value'].'</a>';
            $optData .= '</td>';
            $optData .= '<td id="option-control_'.$array[0].'"style="height:24px;display:none">';
            if ( $row['type'] == 'list' ) {
                $optData .= '<select>';
                foreach ( explode( ',', $row['range'] ) as $opt ) {
                    if ( $row['value'] == $opt ) {
                        $optData .= '<option selected="selected">'.$opt.'</option>';
                    }
                    else {
                        $optData .= '<option>'.$opt.'</option>';
                    }
                }
                $optData .= '</select>';
            }
            elseif ( $row['type'] == 'bool' ) {
                $optData .= '<select>';
                if ( $row['value'] == 'On' ) {
                    $optData .= '<option selected="selected">On</option><option>Off</option>';
                }
                else {
                    $optData .= '<option>On</option><option selected="selected">Off</option>';
                }
                $optData .= '</select>';
            }
            else {
                $optData .= '<input type="text" size="12" value="'.$row['value'].'"/>';
            }
            $optData .= '<input type="button" class="opt-submit" value="Apply"/>';
            $optData .= '<input type="hidden" value="'.($row['default'] ? $row['default'] : 'no value').'"/>';
            $optData .= '<input type="button" class="opt-reset" value="Reset"/>';
            $optData .= '</td>';
        }
        else {
            $optData .= '<td style="height:24px;" id="option-value_'.$array[0].'">';
            $optData .= '<a href="#" style="color:#A0A0A0;" class="option-link_'.$array[0].'">'.($row['default'] ? $row['default'] : 'no value').'</a>';
            $optData .= '</td>';
            $optData .= '<td id="option-control_'.$array[0].'"style="height:24px;display:none">';
            if ( $row['type'] == 'list' ) {
                $optData .= '<select>';
                foreach ( explode( ',', $row['range'] ) as $opt ) {
                    $optData .= '<option>'.$opt.'</option>';
                }
                $optData .= '</select>';
            }
            elseif ( $row['type'] == 'bool' ) {
                $optData .= '<select><option>On</option><option>Off</option></select>';
            }
            else {
                $optData .= '<input type="text" size="12" value="'.$row['default'].'"/>';
            }
            $optData .= '<input type="button" class="opt-submit" value="Apply"/>';
            $optData .= '<input type="hidden" value="'.($row['default'] ? $row['default'] : 'no value').'"/>';
            $optData .= '</td>';
        }
        $optData .= '</tr>';
    }
    return array('data'=>$optData);
}

?>
