#!/bin/sh
eval 'if [ -x /usr/local/cpanel/3rdparty/bin/perl ]; then exec /usr/local/cpanel/3rdparty/bin/perl -x -- $0 ${1+"$@"}; else exec /usr/bin/perl -x -- $0 ${1+"$@"};fi'
if 0;
#!/usr/bin/perl
# CloudLinux LVE Manager VERSION:0.83

BEGIN { unshift @INC, "/usr/local/cpanel"; }

use strict;
use Cpanel::Form    ();
use Cpanel::SafeRun ();
use Data::Dumper    ();

my %FORM    = Cpanel::Form::parseform();
my $user    = $ENV{'REMOTE_USER'}||$ARGV[0];

# pull extensins set and status for a selector version
if ( $FORM{extversion} ) {
    my %data;
    my $version = $FORM{extversion};
    open my $p, "/usr/bin/cl-selector --list-extensions=php --user=$user --version=$version --all|";
    while ( my $line = <$p> ) {
        chomp $line;
        my ( $status, $extension ) = split /\s+/, $line;
        $status = $status eq '+' ? 1 : 0;
        $data{$extension} = $status;
    }
    close $p;
    my $d = Data::Dumper->new([
        [ map +{ title=>$_, status=>$data{$_} }, sort keys %data ]
    ]);
    print "Content-Type: application/json\n\n";
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
    exit(0);
}

# on settings version selector change
elsif ( $FORM{phpversion} and not exists $FORM{optionset} ) {
    
    my $version = check_params( 'version', $FORM{phpversion} );
    unless ( $version ) {
        send_error('Unacceptable version format');
        exit(0);
    }
    
    open my $p, qq#/usr/bin/piniset --user=$user --version=$version --json|#;
    my $data = do { local $/; <$p> };
    close $p;

    print "Content-Type: application/json\n\n";
    if ( $data =~ /(ERROR:.*)$/m) {
        my $d = Data::Dumper->new([ { status => 'error', errmsg => $1 } ]);
        print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
    }
    else {
        # we got data in JSON so we need no more processing
        print $data;
    }
}

# on settings submit 
elsif ( $FORM{phpversion} and exists $FORM{optionset} ) {
    
    my $version = check_params( 'version', $FORM{phpversion} );
    unless ( $version ) {
        send_error('Unacceptable version format');
        exit(0);
    }
    my $optset = q{};
    if ( $FORM{optionset} ) { # not an empty string!
        $optset = check_params( 'paramstring', $FORM{optionset} );
        unless ( $optset ) {
            send_error('Unacceptable options format');
            exit(0);
        }
    }
    open my $p, qq#/usr/bin/piniset --user=$user --version=$version --replace="$optset"|#;
    my $data = do { local $/; <$p> };
    close $p;
    
    print "Content-Type: application/json\n\n";
    my $val;
    if ( $data =~ /(ERROR:.*)$/m) {
        $val = { status => 'error', errmsg => $1 };
    }
    else {
        $val = { status => 'OK' };
    }
    
    my $d = Data::Dumper->new([ $val ]);
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
}

# apply extensions set and return warnings
elsif ( $FORM{selversion} and ! $FORM{setdefaults} ) {
    my %extlist_hash = map { substr($_,5) => $FORM{$_} } grep { index($_,'chbx') == 0 } keys %FORM;
    my $extensions_to_apply = join( q{,}, grep { $extlist_hash{$_} == 1 } keys %extlist_hash );
    open my $p, qq#/usr/bin/cl-selector --interpreter=php --update=$extensions_to_apply --user=$user --version=$FORM{selversion} --with-reload --backup|#;
    my $return_string = do { local $/; <$p> };
    close $p;

    # process returned info
    if ( $return_string ) {

        # unrecoverable error: show nothing but it
        if ( $return_string =~ /(ERROR:.*)$/ ) {
            return [ { errmsg => $1 } ];
        }
        else { # if warnings only update extensions list and show the warnings unchanged
            for my $line ( split( /\n/, $return_string ) ) {

                # get extension itself, what's happened to it and conflicts or dependencies
                my ( $ext, $action, $arg ) = $line =~ /WARN:(\S+)\s(\S+).*?(?:\((.*)\))?\.?$/;

                #$arg ||= q{};

                # if extension was skipped (for conflict), remove it
                $extlist_hash{$ext} = 0 if exists $extlist_hash{$ext} and $action eq 'skipped';

                # if extension was enabled (for dependency), add it
                $extlist_hash{$ext} = 1 if exists $extlist_hash{$ext} and $action eq 'enabled';

                # if extension was left (for dependency), add it
                $extlist_hash{$ext} = 1 if exists $extlist_hash{$ext} and $action eq 'left';
            }
        }
    }
    my @ext = map +{ title=>$_, state=>$extlist_hash{$_} }, sort keys %extlist_hash;
    push @ext, { warnmsg => $return_string } if $return_string;
    my $d = Data::Dumper->new([ \@ext ]);
    print "Content-Type: application/json\n\n";
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
}
# apply defaults for a user
elsif ( $FORM{setdefaults} and $FORM{selversion} ) {
    my %extlist_hash = ();
    open my $p, qq#/usr/bin/cl-selector --set-default-extensions=php --user=$user --version=$FORM{selversion} --with-reload --backup|#;
    my $return_string = do { local $/; <$p> };
    close $p;

    # process returned info
    if ( $return_string ) {

        # unrecoverable error: show nothing but it
        if ( $return_string =~ /(ERROR:.*)$/ ) {
            return [ { errmsg => $1 } ];
        }
        else { # if warnings only update extensions list and show the warnings unchanged

            my @return_data = split( /\n/, $return_string );
            %extlist_hash = map { $_=>1 } split( /,/, pop @return_data );

            # remove from returned data modules list, leaving only warnings
            $return_string = scalar @return_data ? join("\n",@return_data) : undef;

            for my $line ( @return_data ) {

                # get extension itself, what's happened to it and conflicts or dependencies
                my ( $ext, $action, $arg ) = $line =~ /WARN:(\S+)\s(\S+).*?(?:\((.*)\))?\.?$/;

                #$arg ||= q{};

                # if extension was skipped (for conflict), remove it
                $extlist_hash{$ext} = 0 if exists $extlist_hash{$ext} and $action eq 'skipped';

                # if extension was enabled (for dependency), add it
                $extlist_hash{$ext} = 1 if $action eq 'enabled';

                # if extension was left (for dependency), add it
                $extlist_hash{$ext} = 1 if $action eq 'left';
            }
        }
    }

    my @ext = map +{ title=>$_, state=>$extlist_hash{$_} }, sort keys %extlist_hash;
    push @ext, { warnmsg => $return_string } if $return_string;
    my $d = Data::Dumper->new([ \@ext ]);
    print "Content-Type: application/json\n\n";
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
}

sub check_params {
    my ( $type, $data ) = @_;
    
    if ( $type eq 'version' ) {
        if ( $data =~ /(native|\d\.\d)/ ) {
            return $1;
        }
        return;
    }
    elsif ( $type eq 'paramstring' ) {
        if ( $data =~ /((?:[A-Za-z\._\-0-9]+:[A-Za-z\.\_\-0-9\/\\ &|]+,?){1,})/ ) {
            my $filtered = $1;
            $filtered =~ s/(?:\/?passwd|\/?fstab|\/?issue|:\/?proc|\/?sys|(?:cat\s)|(?:ls\s))//g;
            return $filtered;
        }
        return;
    }
}

sub send_error {
    my ( $message ) = @_;
    my $d = Data::Dumper->new([ { status => 'error', errmsg => $message } ]);
    print "Content-Type: application/json\n\n";
    print $d->Terse(1)->Pair(':')->Useqq(1)->Deepcopy(1)->Dump;
}
